@@ -1,27 +1,22 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include <limits.h>
 
 #include "modsecurity.h"
 #include "msc_logging.h"
 #include "msc_util.h"
-#include "pdf_protect.h"
 #include "http_log.h"
 
 #if defined(WITH_LUA)
@@ -47,6 +42,7 @@ void *create_directory_config(apr_pool_t *mp, char *path)
     dcfg->is_enabled = NOT_SET;
 
     dcfg->reqbody_access = NOT_SET;
+    dcfg->reqintercept_oe = NOT_SET;
     dcfg->reqbody_buffering = NOT_SET;
     dcfg->reqbody_inmemory_limit = NOT_SET;
     dcfg->reqbody_limit = NOT_SET;
@@ -58,6 +54,7 @@ void *create_directory_config(apr_pool_t *mp, char *path)
     dcfg->debuglog_fd = NOT_SET_P;
 
     dcfg->of_limit = NOT_SET;
+    dcfg->if_limit_action = NOT_SET;
     dcfg->of_limit_action = NOT_SET;
     dcfg->of_mime_types = NOT_SET_P;
     dcfg->of_mime_types_cleared = NOT_SET;
@@ -103,17 +100,16 @@ void *create_directory_config(apr_pool_t *mp, char *path)
     /* Content injection. */
     dcfg->content_injection_enabled = NOT_SET;
 
-    /* PDF XSS protection. */
-    dcfg->pdfp_enabled = NOT_SET;
-    dcfg->pdfp_secret = NOT_SET_P;
-    dcfg->pdfp_timeout = NOT_SET;
-    dcfg->pdfp_token_name = NOT_SET_P;
-    dcfg->pdfp_only_get = NOT_SET;
-    dcfg->pdfp_method = NOT_SET;
+    /* Stream inspection */
+    dcfg->stream_inbody_inspection = NOT_SET;
+    dcfg->stream_outbody_inspection = NOT_SET;
 
     /* Geo Lookups */
     dcfg->geo = NOT_SET_P;
 
+    /* Gsb Lookups */
+    dcfg->gsb = NOT_SET_P;
+
     /* Cache */
     dcfg->cache_trans = NOT_SET;
     dcfg->cache_trans_incremental = NOT_SET;
@@ -124,6 +120,7 @@ void *create_directory_config(apr_pool_t *mp, char *path)
     dcfg->component_signatures = apr_array_make(mp, 16, sizeof(char *));
 
     dcfg->request_encoding = NOT_SET_P;
+    dcfg->disable_backend_compression = NOT_SET;
 
     return dcfg;
 }
@@ -165,18 +162,40 @@ static void copy_rules_phase(apr_pool_t *mp,
                             char *my_error_msg = NULL;
 
                             int rc = msc_regexec(exceptions[j]->param_data,
-                                rule->actionset->msg, strlen(rule->actionset->msg),
-                                &my_error_msg);
+                                    rule->actionset->msg, strlen(rule->actionset->msg),
+                                    &my_error_msg);
                             if (rc >= 0) copy--;
                         }
                         break;
+                    case RULE_EXCEPTION_REMOVE_TAG :
+                        if ((rule->actionset != NULL)&&(apr_is_empty_table(rule->actionset->actions) == 0)) {
+                            char *my_error_msg = NULL;
+                            const apr_array_header_t *tarr = NULL;
+                            const apr_table_entry_t *telts = NULL;
+                            int i;
+
+                            tarr = apr_table_elts(rule->actionset->actions);
+                            telts = (const apr_table_entry_t*)tarr->elts;
+
+                            for (i = 0; i < tarr->nelts; i++) {
+                                msre_action *action = (msre_action *)telts[i].val;
+                                if(strcmp("tag", action->metadata->name) == 0)  {
+
+                                    int rc = msc_regexec(exceptions[j]->param_data,
+                                            action->param, strlen(action->param),
+                                            &my_error_msg);
+                                    if (rc >= 0) copy--;
+                                }
+                            }
+                        }
+                        break;
                 }
             }
 
             if (copy > 0) {
-                #ifdef DEBUG_CONF
+#ifdef DEBUG_CONF
                 ap_log_perror(APLOG_MARK, APLOG_STARTUP|APLOG_NOERRNO, 0, mp, "Copy rule %pp [id \"%s\"]", rule, rule->actionset->id);
-                #endif
+#endif
 
                 /* Copy the rule. */
                 *(msre_rule **)apr_array_push(child_phase_arr) = rule;
@@ -186,9 +205,9 @@ static void copy_rules_phase(apr_pool_t *mp,
             }
         } else {
             if (mode == 2) {
-                #ifdef DEBUG_CONF
+#ifdef DEBUG_CONF
                 ap_log_perror(APLOG_MARK, APLOG_STARTUP|APLOG_NOERRNO, 0, mp, "Copy chain %pp for rule %pp [id \"%s\"]", rule, rule->chain_starter, rule->chain_starter->actionset->id);
-                #endif
+#endif
 
                 /* Copy the rule (it belongs to the chain we want to include. */
                 *(msre_rule **)apr_array_push(child_phase_arr) = rule;
@@ -259,8 +278,12 @@ void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child)
 
     merged->of_limit = (child->of_limit == NOT_SET
         ? parent->of_limit : child->of_limit);
+    merged->if_limit_action = (child->if_limit_action == NOT_SET
+        ? parent->if_limit_action : child->if_limit_action);
     merged->of_limit_action = (child->of_limit_action == NOT_SET
         ? parent->of_limit_action : child->of_limit_action);
+    merged->reqintercept_oe = (child->reqintercept_oe == NOT_SET
+        ? parent->reqintercept_oe : child->reqintercept_oe);
 
     if (child->of_mime_types != NOT_SET_P) {
         /* Child added to the table */
@@ -446,24 +469,20 @@ void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child)
     merged->content_injection_enabled = (child->content_injection_enabled == NOT_SET
         ? parent->content_injection_enabled : child->content_injection_enabled);
 
-    /* PDF XSS protection. */
-    merged->pdfp_enabled = (child->pdfp_enabled == NOT_SET
-        ? parent->pdfp_enabled : child->pdfp_enabled);
-    merged->pdfp_secret = (child->pdfp_secret == NOT_SET_P
-        ? parent->pdfp_secret : child->pdfp_secret);
-    merged->pdfp_timeout = (child->pdfp_timeout == NOT_SET
-        ? parent->pdfp_timeout : child->pdfp_timeout);
-    merged->pdfp_token_name = (child->pdfp_token_name == NOT_SET_P
-        ? parent->pdfp_token_name : child->pdfp_token_name);
-    merged->pdfp_only_get = (child->pdfp_only_get == NOT_SET
-        ? parent->pdfp_only_get : child->pdfp_only_get);
-    merged->pdfp_method = (child->pdfp_method == NOT_SET
-        ? parent->pdfp_method : child->pdfp_method);
+    /* Stream inspection */
+    merged->stream_inbody_inspection = (child->stream_inbody_inspection == NOT_SET
+        ? parent->stream_inbody_inspection : child->stream_inbody_inspection);
+    merged->stream_outbody_inspection = (child->stream_outbody_inspection == NOT_SET
+        ? parent->stream_outbody_inspection : child->stream_outbody_inspection);
 
     /* Geo Lookup */
     merged->geo = (child->geo == NOT_SET_P
         ? parent->geo : child->geo);
 
+    /* Gsb Lookup */
+    merged->gsb = (child->gsb == NOT_SET_P
+        ? parent->gsb : child->gsb);
+
     /* Cache */
     merged->cache_trans = (child->cache_trans == NOT_SET
         ? parent->cache_trans : child->cache_trans);
@@ -483,6 +502,9 @@ void *merge_directory_configs(apr_pool_t *mp, void *_parent, void *_child)
     merged->request_encoding = (child->request_encoding == NOT_SET_P
         ? parent->request_encoding : child->request_encoding);
 
+    merged->disable_backend_compression = (child->disable_backend_compression == NOT_SET
+        ? parent->disable_backend_compression : child->disable_backend_compression);
+
     return merged;
 }
 
@@ -499,6 +521,7 @@ void init_directory_config(directory_config *dcfg)
     if (dcfg->is_enabled == NOT_SET) dcfg->is_enabled = 0;
 
     if (dcfg->reqbody_access == NOT_SET) dcfg->reqbody_access = 0;
+    if (dcfg->reqintercept_oe == NOT_SET) dcfg->reqintercept_oe = 0;
     if (dcfg->reqbody_buffering == NOT_SET) dcfg->reqbody_buffering = REQUEST_BODY_FORCEBUF_OFF;
     if (dcfg->reqbody_inmemory_limit == NOT_SET)
         dcfg->reqbody_inmemory_limit = REQUEST_BODY_DEFAULT_INMEMORY_LIMIT;
@@ -506,6 +529,7 @@ void init_directory_config(directory_config *dcfg)
     if (dcfg->reqbody_no_files_limit == NOT_SET) dcfg->reqbody_no_files_limit = REQUEST_BODY_NO_FILES_DEFAULT_LIMIT;
     if (dcfg->resbody_access == NOT_SET) dcfg->resbody_access = 0;
     if (dcfg->of_limit == NOT_SET) dcfg->of_limit = RESPONSE_BODY_DEFAULT_LIMIT;
+    if (dcfg->if_limit_action == NOT_SET) dcfg->if_limit_action = REQUEST_BODY_LIMIT_ACTION_REJECT;
     if (dcfg->of_limit_action == NOT_SET) dcfg->of_limit_action = RESPONSE_BODY_LIMIT_ACTION_REJECT;
 
     if (dcfg->of_mime_types == NOT_SET_P) {
@@ -553,17 +577,16 @@ void init_directory_config(directory_config *dcfg)
     /* Content injection. */
     if (dcfg->content_injection_enabled == NOT_SET) dcfg->content_injection_enabled = 0;
 
-    /* PDF XSS protection. */
-    if (dcfg->pdfp_enabled == NOT_SET) dcfg->pdfp_enabled = 0;
-    if (dcfg->pdfp_secret == NOT_SET_P) dcfg->pdfp_secret = NULL;
-    if (dcfg->pdfp_timeout == NOT_SET) dcfg->pdfp_timeout = 10;
-    if (dcfg->pdfp_token_name == NOT_SET_P) dcfg->pdfp_token_name = "PDFPTOKEN";
-    if (dcfg->pdfp_only_get == NOT_SET) dcfg->pdfp_only_get = 1;
-    if (dcfg->pdfp_method == NOT_SET) dcfg->pdfp_method = PDF_PROTECT_METHOD_TOKEN_REDIRECTION;
+    /* Stream inspection */
+    if (dcfg->stream_inbody_inspection == NOT_SET) dcfg->stream_inbody_inspection = 0;
+    if (dcfg->stream_outbody_inspection == NOT_SET) dcfg->stream_outbody_inspection = 0;
 
     /* Geo Lookup */
     if (dcfg->geo == NOT_SET_P) dcfg->geo = NULL;
 
+    /* Gsb Lookup */
+    if (dcfg->gsb == NOT_SET_P) dcfg->gsb = NULL;
+
     /* Cache */
     if (dcfg->cache_trans == NOT_SET) dcfg->cache_trans = MODSEC_CACHE_DISABLED;
     if (dcfg->cache_trans_incremental == NOT_SET) dcfg->cache_trans_incremental = 0;
@@ -573,6 +596,7 @@ void init_directory_config(directory_config *dcfg)
 
     if (dcfg->request_encoding == NOT_SET_P) dcfg->request_encoding = NULL;
 
+    if (dcfg->disable_backend_compression == NOT_SET) dcfg->disable_backend_compression = 0;
 }
 
 /**
@@ -825,7 +849,7 @@ static const char *add_marker(cmd_parms *cmd, directory_config *dcfg,
  *
  */
 static const char *update_rule_action(cmd_parms *cmd, directory_config *dcfg,
-                                      const char *p1, const char *p2)
+                                      const char *p1, const char *p2, int offset)
 {
     char *my_error_msg = NULL;
     msre_rule *rule = NULL;
@@ -844,7 +868,7 @@ static const char *update_rule_action(cmd_parms *cmd, directory_config *dcfg,
     #endif
 
     /* Fetch the rule */
-    rule = msre_ruleset_fetch_rule(ruleset, p1);
+    rule = msre_ruleset_fetch_rule(ruleset, p1, offset);
     if (rule == NULL) {
         #ifdef DEBUG_CONF
         ap_log_perror(APLOG_MARK, APLOG_STARTUP|APLOG_NOERRNO, 0, cmd->pool,
@@ -923,7 +947,7 @@ static const char *cmd_marker(cmd_parms *cmd, void *_dcfg, const char *p1)
 }
 
 static const char *cmd_argument_separator(cmd_parms *cmd, void *_dcfg,
-                                          const char *p1)
+        const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
 
@@ -942,12 +966,12 @@ static const char *cmd_audit_engine(cmd_parms *cmd, void *_dcfg, const char *p1)
 
     if (strcasecmp(p1, "On") == 0) dcfg->auditlog_flag = AUDITLOG_ON;
     else
-    if (strcasecmp(p1, "Off") == 0) dcfg->auditlog_flag = AUDITLOG_OFF;
-    else
-    if (strcasecmp(p1, "RelevantOnly") == 0) dcfg->auditlog_flag = AUDITLOG_RELEVANT;
-    else
-    return (const char *)apr_psprintf(cmd->pool,
-        "ModSecurity: Unrecognised parameter value for SecAuditEngine: %s", p1);
+        if (strcasecmp(p1, "Off") == 0) dcfg->auditlog_flag = AUDITLOG_OFF;
+        else
+            if (strcasecmp(p1, "RelevantOnly") == 0) dcfg->auditlog_flag = AUDITLOG_RELEVANT;
+            else
+                return (const char *)apr_psprintf(cmd->pool,
+                        "ModSecurity: Unrecognised parameter value for SecAuditEngine: %s", p1);
 
     return NULL;
 }
@@ -965,7 +989,7 @@ static const char *cmd_audit_log(cmd_parms *cmd, void *_dcfg, const char *p1)
         pipe_log = ap_open_piped_log(cmd->pool, pipe_name);
         if (pipe_log == NULL) {
             return apr_psprintf(cmd->pool, "ModSecurity: Failed to open the audit log pipe: %s",
-                pipe_name);
+                    pipe_name);
         }
         dcfg->auditlog_fd = ap_piped_log_write_fd(pipe_log);
     }
@@ -974,12 +998,12 @@ static const char *cmd_audit_log(cmd_parms *cmd, void *_dcfg, const char *p1)
         apr_status_t rc;
 
         rc = apr_file_open(&dcfg->auditlog_fd, file_name,
-            APR_WRITE | APR_APPEND | APR_CREATE | APR_BINARY,
-            CREATEMODE, cmd->pool);
+                APR_WRITE | APR_APPEND | APR_CREATE | APR_BINARY,
+                CREATEMODE, cmd->pool);
 
         if (rc != APR_SUCCESS) {
             return apr_psprintf(cmd->pool, "ModSecurity: Failed to open the audit log file: %s",
-                file_name);
+                    file_name);
         }
     }
 
@@ -1003,7 +1027,7 @@ static const char *cmd_audit_log2(cmd_parms *cmd, void *_dcfg, const char *p1)
         pipe_log = ap_open_piped_log(cmd->pool, pipe_name);
         if (pipe_log == NULL) {
             return apr_psprintf(cmd->pool, "ModSecurity: Failed to open the secondary audit log pipe: %s",
-                pipe_name);
+                    pipe_name);
         }
         dcfg->auditlog2_fd = ap_piped_log_write_fd(pipe_log);
     }
@@ -1012,12 +1036,12 @@ static const char *cmd_audit_log2(cmd_parms *cmd, void *_dcfg, const char *p1)
         apr_status_t rc;
 
         rc = apr_file_open(&dcfg->auditlog2_fd, file_name,
-            APR_WRITE | APR_APPEND | APR_CREATE | APR_BINARY,
-            CREATEMODE, cmd->pool);
+                APR_WRITE | APR_APPEND | APR_CREATE | APR_BINARY,
+                CREATEMODE, cmd->pool);
 
         if (rc != APR_SUCCESS) {
             return apr_psprintf(cmd->pool, "ModSecurity: Failed to open the secondary audit log file: %s",
-                file_name);
+                    file_name);
         }
     }
 
@@ -1025,7 +1049,7 @@ static const char *cmd_audit_log2(cmd_parms *cmd, void *_dcfg, const char *p1)
 }
 
 static const char *cmd_audit_log_parts(cmd_parms *cmd, void *_dcfg,
-                                       const char *p1)
+        const char *p1)
 {
     directory_config *dcfg = _dcfg;
 
@@ -1038,7 +1062,7 @@ static const char *cmd_audit_log_parts(cmd_parms *cmd, void *_dcfg,
 }
 
 static const char *cmd_audit_log_relevant_status(cmd_parms *cmd, void *_dcfg,
-                                                 const char *p1)
+        const char *p1)
 {
     directory_config *dcfg = _dcfg;
 
@@ -1051,22 +1075,22 @@ static const char *cmd_audit_log_relevant_status(cmd_parms *cmd, void *_dcfg,
 }
 
 static const char *cmd_audit_log_type(cmd_parms *cmd, void *_dcfg,
-                                      const char *p1)
+        const char *p1)
 {
     directory_config *dcfg = _dcfg;
 
     if (strcasecmp(p1, "Serial") == 0) dcfg->auditlog_type = AUDITLOG_SERIAL;
     else
-    if (strcasecmp(p1, "Concurrent") == 0) dcfg->auditlog_type = AUDITLOG_CONCURRENT;
-    else
-    return (const char *)apr_psprintf(cmd->pool,
-        "ModSecurity: Unrecognised parameter value for SecAuditLogType: %s", p1);
+        if (strcasecmp(p1, "Concurrent") == 0) dcfg->auditlog_type = AUDITLOG_CONCURRENT;
+        else
+            return (const char *)apr_psprintf(cmd->pool,
+                    "ModSecurity: Unrecognised parameter value for SecAuditLogType: %s", p1);
 
     return NULL;
 }
 
 static const char *cmd_audit_log_dirmode(cmd_parms *cmd, void *_dcfg,
-                                         const char *p1)
+        const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
 
@@ -1088,7 +1112,7 @@ static const char *cmd_audit_log_dirmode(cmd_parms *cmd, void *_dcfg,
 }
 
 static const char *cmd_audit_log_filemode(cmd_parms *cmd, void *_dcfg,
-                                          const char *p1)
+        const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
 
@@ -1292,6 +1316,14 @@ static const char *cmd_default_action(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
+static const char *cmd_disable_backend_compression(cmd_parms *cmd, void *_dcfg, int flag)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+    if (dcfg == NULL) return NULL;
+    dcfg->disable_backend_compression = flag;
+    return NULL;
+}
+
 static const char *cmd_guardian_log(cmd_parms *cmd, void *_dcfg,
                                     const char *p1, const char *p2)
 {
@@ -1343,6 +1375,43 @@ static const char *cmd_guardian_log(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
+/*
+* \brief Add SecStreamInBodyInspection configuration option
+*
+* \param cmd Pointer to configuration data
+* \param _dcfg Pointer to directory configuration
+* \param p1 Pointer to configuration option
+*
+* \retval NULL On failure
+* \retval apr_psprintf On Success
+*/
+static const char *cmd_stream_inbody_inspection(cmd_parms *cmd, void *_dcfg, int flag)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+    if (dcfg == NULL) return NULL;
+    dcfg->stream_inbody_inspection = flag;
+    return NULL;
+}
+
+
+/*
+* \brief Add SecStreamOutBodyInspection configuration option
+*
+* \param cmd Pointer to configuration data
+* \param _dcfg Pointer to directory configuration
+* \param p1 Pointer to configuration option
+*
+* \retval NULL On failure
+* \retval apr_psprintf On Success
+*/
+static const char *cmd_stream_outbody_inspection(cmd_parms *cmd, void *_dcfg, int flag)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+    if (dcfg == NULL) return NULL;
+    dcfg->stream_outbody_inspection = flag;
+    return NULL;
+}
+
 /*
 * \brief Add SecReadStateLimit configuration option
 *
@@ -1354,7 +1423,7 @@ static const char *cmd_guardian_log(cmd_parms *cmd, void *_dcfg,
 * \retval apr_psprintf On Success
 */
 static const char *cmd_conn_read_state_limit(cmd_parms *cmd, void *_dcfg,
-                                                   const char *p1)
+        const char *p1)
 {
     directory_config *dcfg = (directory_config *)_dcfg;
     long int limit;
@@ -1371,6 +1440,35 @@ static const char *cmd_conn_read_state_limit(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
+/*
+* \brief Add SecWriteStateLimit configuration option
+*
+* \param cmd Pointer to configuration data
+* \param _dcfg Pointer to directory configuration
+* \param p1 Pointer to configuration option
+*
+* \retval NULL On failure
+* \retval apr_psprintf On Success
+*/
+static const char *cmd_conn_write_state_limit(cmd_parms *cmd, void *_dcfg,
+        const char *p1)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+    long int limit;
+
+    if (dcfg == NULL) return NULL;
+
+    limit = strtol(p1, NULL, 10);
+    if ((limit == LONG_MAX)||(limit == LONG_MIN)||(limit <= 0)) {
+        return apr_psprintf(cmd->pool, "ModSecurity: Invalid value for SecWriteStateLimit: %s", p1);
+    }
+
+    conn_write_state_limit = limit;
+
+    return NULL;
+}
+
+
 static const char *cmd_request_body_inmemory_limit(cmd_parms *cmd, void *_dcfg,
                                                    const char *p1)
 {
@@ -1440,6 +1538,22 @@ static const char *cmd_request_body_access(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
+static const char *cmd_request_intercept_on_error(cmd_parms *cmd, void *_dcfg,
+                                           const char *p1)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+    if (dcfg == NULL) return NULL;
+
+    if (strcasecmp(p1, "on") == 0) dcfg->reqintercept_oe = 1;
+    else
+    if (strcasecmp(p1, "off") == 0) dcfg->reqintercept_oe = 0;
+    else
+    return apr_psprintf(cmd->pool, "ModSecurity: Invalid value for SecInterceptOnError: %s", p1);
+
+    return NULL;
+}
+
+
 static const char *cmd_request_encoding(cmd_parms *cmd, void *_dcfg,
                                         const char *p1)
 {
@@ -1494,6 +1608,11 @@ static const char *cmd_response_body_limit_action(cmd_parms *cmd, void *_dcfg,
     directory_config *dcfg = (directory_config *)_dcfg;
     if (dcfg == NULL) return NULL;
 
+    if (dcfg->is_enabled == MODSEC_DETECTION_ONLY)  {
+        dcfg->of_limit_action = RESPONSE_BODY_LIMIT_ACTION_PARTIAL;
+        return NULL;
+    }
+
     if (strcasecmp(p1, "ProcessPartial") == 0) dcfg->of_limit_action = RESPONSE_BODY_LIMIT_ACTION_PARTIAL;
     else
     if (strcasecmp(p1, "Reject") == 0) dcfg->of_limit_action = RESPONSE_BODY_LIMIT_ACTION_REJECT;
@@ -1503,6 +1622,26 @@ static const char *cmd_response_body_limit_action(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
+static const char *cmd_resquest_body_limit_action(cmd_parms *cmd, void *_dcfg,
+                                                  const char *p1)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+    if (dcfg == NULL) return NULL;
+
+    if (dcfg->is_enabled == MODSEC_DETECTION_ONLY)  {
+        dcfg->if_limit_action = REQUEST_BODY_LIMIT_ACTION_PARTIAL;
+        return NULL;
+    }
+
+    if (strcasecmp(p1, "ProcessPartial") == 0) dcfg->if_limit_action = REQUEST_BODY_LIMIT_ACTION_PARTIAL;
+    else
+    if (strcasecmp(p1, "Reject") == 0) dcfg->if_limit_action = REQUEST_BODY_LIMIT_ACTION_REJECT;
+    else
+    return apr_psprintf(cmd->pool, "ModSecurity: Invalid value for SecRequestBodyLimitAction: %s", p1);
+
+    return NULL;
+}
+
 static const char *cmd_response_body_mime_type(cmd_parms *cmd, void *_dcfg,
                                                const char *_p1)
 {
@@ -1536,8 +1675,25 @@ static const char *cmd_response_body_mime_types_clear(cmd_parms *cmd,
     return NULL;
 }
 
+/*
+* \brief Add SecRuleUpdateTargetById
+*
+* \param cmd Pointer to configuration data
+* \param _dcfg Pointer to directory configuration
+* \param p1 Pointer to configuration option
+* \param p2 Pointer to configuration option
+* \param p3 Pointer to configuration option
+*
+* \retval NULL On failure|Success
+*/
+static const char *cmd_rule_update_target_by_id(cmd_parms *cmd, void *_dcfg,
+        const char *p1, const char *p2, const char *p3)
+{
+    return update_rule_target(cmd, (directory_config *)_dcfg, NULL, p1, p2, p3);
+}
+
 static const char *cmd_rule(cmd_parms *cmd, void *_dcfg,
-                            const char *p1, const char *p2, const char *p3)
+        const char *p1, const char *p2, const char *p3)
 {
     return add_rule(cmd, (directory_config *)_dcfg, RULE_TYPE_NORMAL, p1, p2, p3);
 }
@@ -1551,8 +1707,11 @@ static const char *cmd_rule_engine(cmd_parms *cmd, void *_dcfg, const char *p1)
     else
     if (strcasecmp(p1, "off") == 0) dcfg->is_enabled = MODSEC_DISABLED;
     else
-    if (strcasecmp(p1, "detectiononly") == 0) dcfg->is_enabled = MODSEC_DETECTION_ONLY;
-    else
+    if (strcasecmp(p1, "detectiononly") == 0) {
+        dcfg->is_enabled = MODSEC_DETECTION_ONLY;
+        dcfg->of_limit_action = RESPONSE_BODY_LIMIT_ACTION_PARTIAL;
+        dcfg->if_limit_action = REQUEST_BODY_LIMIT_ACTION_PARTIAL;
+    } else
     return apr_psprintf(cmd->pool, "ModSecurity: Invalid value for SecRuleEngine: %s", p1);
 
     return NULL;
@@ -1595,6 +1754,31 @@ static const char *cmd_rule_remove_by_id(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
+static const char *cmd_rule_remove_by_tag(cmd_parms *cmd, void *_dcfg,
+                                          const char *p1)
+{
+    directory_config *dcfg = (directory_config *)_dcfg;
+    rule_exception *re = apr_pcalloc(cmd->pool, sizeof(rule_exception));
+    if (dcfg == NULL) return NULL;
+
+    re->type = RULE_EXCEPTION_REMOVE_TAG;
+    re->param = p1;
+    re->param_data = msc_pregcomp(cmd->pool, p1, 0, NULL, NULL);
+    if (re->param_data == NULL) {
+        return apr_psprintf(cmd->pool, "ModSecurity: Invalid regular expression: %s", p1);
+    }
+    *(rule_exception **)apr_array_push(dcfg->rule_exceptions) = re;
+
+    /* Remove the corresponding rules from the context straight away. */
+    msre_ruleset_rule_remove_with_exception(dcfg->ruleset, re);
+
+    #ifdef DEBUG_CONF
+    ap_log_perror(APLOG_MARK, APLOG_STARTUP|APLOG_NOERRNO, 0, cmd->pool, "Added exception %pp (%d %s) to dcfg %pp.", re, re->type, re->param, dcfg);
+    #endif
+
+    return NULL;
+}
+
 static const char *cmd_rule_remove_by_msg(cmd_parms *cmd, void *_dcfg,
                                           const char *p1)
 {
@@ -1621,9 +1805,25 @@ static const char *cmd_rule_remove_by_msg(cmd_parms *cmd, void *_dcfg,
 }
 
 static const char *cmd_rule_update_action_by_id(cmd_parms *cmd, void *_dcfg,
-                                                const char *p1, const char *p2)
+        const char *p1, const char *p2)
 {
-    return update_rule_action(cmd, (directory_config *)_dcfg, p1, p2);
+    int offset = 0, rule_id = atoi(p1);
+    char *opt = strchr(p1,':');
+    char *savedptr = NULL;
+    char *param = apr_pstrdup(cmd->pool, p1);
+
+    if ((rule_id == LONG_MAX)||(rule_id == LONG_MIN)||(rule_id <= 0)) {
+        return apr_psprintf(cmd->pool, "ModSecurity: Invalid value for ID for update action: %s", p1);
+    }
+
+    if(opt != NULL) {
+        opt++;
+        offset = atoi(opt);
+        opt = apr_strtok(param,":", &savedptr);
+        return update_rule_action(cmd, (directory_config *)_dcfg, (const char *)opt, p2, offset);
+    }
+
+    return update_rule_action(cmd, (directory_config *)_dcfg, p1, p2, offset);
 }
 
 static const char *cmd_server_signature(cmd_parms *cmd, void *_dcfg,
@@ -1731,7 +1931,6 @@ static const char *cmd_web_app_id(cmd_parms *cmd, void *_dcfg, const char *p1)
     return NULL;
 }
 
-
 /* PCRE Limits */
 
 static const char *cmd_pcre_match_limit(cmd_parms *cmd,
@@ -1773,82 +1972,6 @@ static const char *cmd_pcre_match_limit_recursion(cmd_parms *cmd,
 }
 
 
-
-/* -- PDF Protection configuration -- */
-
-static const char *cmd_pdf_protect(cmd_parms *cmd, void *_dcfg, int flag)
-{
-    directory_config *dcfg = (directory_config *)_dcfg;
-    if (dcfg == NULL) return NULL;
-
-    dcfg->pdfp_enabled = flag;
-
-    return NULL;
-}
-
-static const char *cmd_pdf_protect_secret(cmd_parms *cmd, void *_dcfg,
-                                          const char *p1)
-{
-    directory_config *dcfg = (directory_config *)_dcfg;
-    if (dcfg == NULL) return NULL;
-
-    dcfg->pdfp_secret = p1;
-
-    return NULL;
-}
-
-static const char *cmd_pdf_protect_timeout(cmd_parms *cmd, void *_dcfg,
-                                           const char *p1)
-{
-    directory_config *dcfg = (directory_config *)_dcfg;
-    if (dcfg == NULL) return NULL;
-
-    dcfg->pdfp_timeout = atoi(p1);
-
-    return NULL;
-}
-
-static const char *cmd_pdf_protect_token_name(cmd_parms *cmd, void *_dcfg,
-                                              const char *p1)
-{
-    directory_config *dcfg = (directory_config *)_dcfg;
-    if (dcfg == NULL) return NULL;
-
-    dcfg->pdfp_token_name = p1;
-
-    return NULL;
-}
-
-static const char *cmd_pdf_protect_intercept_get_only(cmd_parms *cmd,
-                                                      void *_dcfg, int flag)
-{
-    directory_config *dcfg = (directory_config *)_dcfg;
-    if (dcfg == NULL) return NULL;
-
-    dcfg->pdfp_only_get = flag;
-
-    return NULL;
-}
-
-static const char *cmd_pdf_protect_method(cmd_parms *cmd, void *_dcfg,
-                                          const char *p1)
-{
-    directory_config *dcfg = (directory_config *)_dcfg;
-    if (dcfg == NULL) return NULL;
-
-    if (strcasecmp(p1, "TokenRedirection") == 0) {
-        dcfg->pdfp_method = PDF_PROTECT_METHOD_TOKEN_REDIRECTION;
-    } else
-    if (strcasecmp(p1, "ForcedDownload") == 0) {
-        dcfg->pdfp_method = PDF_PROTECT_METHOD_FORCED_DOWNLOAD;
-    } else {
-        return (const char *)apr_psprintf(cmd->pool,
-            "ModSecurity: Unrecognised parameter value for SecPdfProtectMethod: %s", p1);
-    }
-
-    return NULL;
-}
-
 /* -- Geo Lookup configuration -- */
 
 static const char *cmd_geo_lookup_db(cmd_parms *cmd, void *_dcfg,
@@ -1866,6 +1989,22 @@ static const char *cmd_geo_lookup_db(cmd_parms *cmd, void *_dcfg,
     return NULL;
 }
 
+/* Google safe browsing */
+
+static const char *cmd_gsb_mal_lookup_db(cmd_parms *cmd, void *_dcfg,
+                                     const char *p1)
+{
+    const char *filename = resolve_relative_path(cmd->pool, cmd->directive->filename, p1);
+    char *error_msg;
+    directory_config *dcfg = (directory_config *)_dcfg;
+    if (dcfg == NULL) return NULL;
+
+    if (gsb_mal_init(dcfg, filename, &error_msg) <= 0) {
+        return error_msg;
+    }
+
+    return NULL;
+}
 
 /* -- Cache -- */
 
@@ -2099,6 +2238,22 @@ const command_rec module_directives[] = {
         "On or Off"
     ),
 
+    AP_INIT_FLAG (
+        "SecStreamOutBodyInspection",
+        cmd_stream_outbody_inspection,
+        NULL,
+        CMD_SCOPE_ANY,
+        "On or Off"
+    ),
+
+    AP_INIT_FLAG (
+        "SecStreamInBodyInspection",
+        cmd_stream_inbody_inspection,
+        NULL,
+        CMD_SCOPE_ANY,
+        "On or Off"
+    ),
+
     AP_INIT_TAKE1 (
         "SecCookieFormat",
         cmd_cookie_format,
@@ -2140,6 +2295,22 @@ const command_rec module_directives[] = {
         "default action list"
     ),
 
+    AP_INIT_FLAG (
+        "SecDisableBackendCompression",
+        cmd_disable_backend_compression,
+        NULL,
+        CMD_SCOPE_ANY,
+        "When set to On, removes the compression headers from the backend requests."
+    ),
+
+    AP_INIT_TAKE1 (
+        "SecGsbLookupDB",
+        cmd_gsb_mal_lookup_db,
+        NULL,
+        RSRC_CONF,
+        "database google safe browsing"
+    ),
+
     AP_INIT_TAKE1 (
         "SecGeoLookupDB",
         cmd_geo_lookup_db,
@@ -2180,54 +2351,6 @@ const command_rec module_directives[] = {
         "PCRE match limit recursion"
     ),
 
-    AP_INIT_FLAG (
-        "SecPdfProtect",
-        cmd_pdf_protect,
-        NULL,
-        RSRC_CONF,
-        "enable PDF protection module."
-    ),
-
-    AP_INIT_TAKE1 (
-        "SecPdfProtectSecret",
-        cmd_pdf_protect_secret,
-        NULL,
-        RSRC_CONF,
-        "secret that will be used to construct protection tokens."
-    ),
-
-    AP_INIT_TAKE1 (
-        "SecPdfProtectTimeout",
-        cmd_pdf_protect_timeout,
-        NULL,
-        RSRC_CONF,
-        "duration for which protection tokens will be valid."
-    ),
-
-    AP_INIT_TAKE1 (
-        "SecPdfProtectTokenName",
-        cmd_pdf_protect_token_name,
-        NULL,
-        RSRC_CONF,
-        "name of the protection token. The name 'PDFTOKEN' is used by default."
-    ),
-
-    AP_INIT_FLAG (
-        "SecPdfProtectInterceptGETOnly",
-        cmd_pdf_protect_intercept_get_only,
-        NULL,
-        RSRC_CONF,
-        "whether or not to intercept only GET and HEAD requess. Defaults to true."
-    ),
-
-    AP_INIT_TAKE1 (
-        "SecPdfProtectMethod",
-        cmd_pdf_protect_method,
-        NULL,
-        RSRC_CONF,
-        "protection method to use. Can be 'TokenRedirection' (default) or 'ForcedDownload'"
-    ),
-
     AP_INIT_TAKE1 (
         "SecRequestBodyAccess",
         cmd_request_body_access,
@@ -2236,6 +2359,14 @@ const command_rec module_directives[] = {
         "On or Off"
     ),
 
+    AP_INIT_TAKE1 (
+        "SecInterceptOnError",
+        cmd_request_intercept_on_error,
+        NULL,
+        CMD_SCOPE_ANY,
+        "On or Off"
+    ),
+
     AP_INIT_TAKE1 (
         "SecReadStateLimit",
         cmd_conn_read_state_limit,
@@ -2244,6 +2375,14 @@ const command_rec module_directives[] = {
         "maximum number of threads in READ_BUSY state per ip address"
     ),
 
+    AP_INIT_TAKE1 (
+        "SecWriteStateLimit",
+        cmd_conn_write_state_limit,
+        NULL,
+        CMD_SCOPE_ANY,
+        "maximum number of threads in WRITE_BUSY state per ip address"
+    ),
+
     AP_INIT_TAKE1 (
         "SecRequestBodyInMemoryLimit",
         cmd_request_body_inmemory_limit,
@@ -2300,6 +2439,14 @@ const command_rec module_directives[] = {
         "what happens when the response body limit is reached"
     ),
 
+    AP_INIT_TAKE1 (
+        "SecRequestBodyLimitAction",
+        cmd_resquest_body_limit_action,
+        NULL,
+        CMD_SCOPE_ANY,
+        "what happens when the request body limit is reached"
+    ),
+
     AP_INIT_ITERATE (
         "SecResponseBodyMimeType",
         cmd_response_body_mime_type,
@@ -2356,6 +2503,14 @@ const command_rec module_directives[] = {
         "rule ID for removal"
     ),
 
+    AP_INIT_ITERATE (
+        "SecRuleRemoveByTag",
+        cmd_rule_remove_by_tag,
+        NULL,
+        CMD_SCOPE_ANY,
+        "rule tag for removal"
+    ),
+
     AP_INIT_ITERATE (
         "SecRuleRemoveByMsg",
         cmd_rule_remove_by_msg,
@@ -2372,6 +2527,14 @@ const command_rec module_directives[] = {
         "updated action list"
     ),
 
+    AP_INIT_TAKE23 (
+        "SecRuleUpdateTargetById",
+        cmd_rule_update_target_by_id,
+        NULL,
+        CMD_SCOPE_ANY,
+        "updated target list"
+    ),
+
     AP_INIT_TAKE1 (
         "SecServerSignature",
         cmd_server_signature,
