@@ -1,27 +1,22 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include <util_filter.h>
 
 #include "modsecurity.h"
 #include "apache2.h"
 
-
 /* -- Input filter -- */
 
 #if 0
@@ -89,7 +84,7 @@ apr_status_t input_filter(ap_filter_t *f, apr_bucket_brigade *bb_out,
         return APR_EGENERAL;
     }
 
-    if (chunk) {
+    if (chunk && !msr->txcfg->stream_inbody_inspection) {
         /* Copy the data we received in the chunk */
         bucket = apr_bucket_heap_create(chunk->data, chunk->length, NULL,
             f->r->connection->bucket_alloc);
@@ -117,6 +112,18 @@ apr_status_t input_filter(ap_filter_t *f, apr_bucket_brigade *bb_out,
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Input filter: Forwarded %" APR_SIZE_T_FMT " bytes.", chunk->length);
         }
+    } else if (msr->stream_input_data != NULL) {
+
+        bucket = apr_bucket_heap_create(msr->stream_input_data, msr->stream_input_length, NULL,
+                f->r->connection->bucket_alloc);
+
+        if (bucket == NULL) return APR_EGENERAL;
+        APR_BRIGADE_INSERT_TAIL(bb_out, bucket);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Input stream filter: Forwarded %" APR_SIZE_T_FMT " bytes.", msr->stream_input_length);
+        }
+
     }
 
     if (rc == 0) {
@@ -188,6 +195,9 @@ apr_status_t read_request_body(modsec_rec *msr, char **error_msg) {
              *      too large and APR_EGENERAL when the client disconnects.
              */
             switch(rc) {
+                case APR_EOF :
+                    *error_msg = apr_psprintf(msr->mp, "Error reading request body: %s", get_apr_error(msr->mp, rc));
+                    return -6;
                 case APR_TIMEUP :
                     *error_msg = apr_psprintf(msr->mp, "Error reading request body: %s", get_apr_error(msr->mp, rc));
                     return -4;
@@ -226,24 +236,70 @@ apr_status_t read_request_body(modsec_rec *msr, char **error_msg) {
 
             /* Check request body limit (should only trigger on chunked requests). */
             if (msr->reqbody_length + buflen > (apr_size_t)msr->txcfg->reqbody_limit) {
-                *error_msg = apr_psprintf(msr->mp, "Request body is larger than the "
-                    "configured limit (%ld).", msr->txcfg->reqbody_limit);
-                return -5;
+                if((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_REJECT)) {
+                    *error_msg = apr_psprintf(msr->mp, "Request body is larger than the "
+                            "configured limit (%ld).", msr->txcfg->reqbody_limit);
+                    return -5;
+                } else if((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)) {
+
+                    *error_msg = apr_psprintf(msr->mp, "Request body is larger than the "
+                            "configured limit (%ld).", msr->txcfg->reqbody_limit);
+
+                    seen_eos = 1;
+                    buflen = (msr->txcfg->reqbody_limit - msr->reqbody_length);
+
+                } else if ((msr->txcfg->is_enabled == MODSEC_DETECTION_ONLY) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)){
+
+                    seen_eos = 1;
+                    buflen = (msr->txcfg->reqbody_limit - msr->reqbody_length);
+
+                } else  {
+
+                    *error_msg = apr_psprintf(msr->mp, "Request body is larger than the "
+                            "configured limit (%ld).", msr->txcfg->reqbody_limit);
+
+                    return -5;
+                }
             }
 
+            if (msr->txcfg->stream_inbody_inspection == 1)   {
+                msr->stream_input_length+=buflen;
+                modsecurity_request_body_to_stream(msr, buf, buflen, error_msg);
+            }
+
+            msr->reqbody_length += buflen;
+
             if (buflen != 0) {
                 int rcbs = modsecurity_request_body_store(msr, buf, buflen, error_msg);
                 if (rcbs < 0) {
                     if (rcbs == -5) {
-                        *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
-                            "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
-                        return -5;
+                        if((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_REJECT)) {
+                            *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
+                                    "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
+                            return -5;
+                        } else if ((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)) {
+                            *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
+                                    "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
+                            seen_eos = 1;
+
+                        } else if ((msr->txcfg->is_enabled == MODSEC_DETECTION_ONLY) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)) {
+                            *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
+                                    "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
+                            seen_eos = 1;
+
+                        } else {
+                            *error_msg = apr_psprintf(msr->mp, "Request body no files data length is larger than the "
+                                    "configured limit (%ld).", msr->txcfg->reqbody_no_files_limit);
+                            return -5;
+                        }
                     }
 
-                    return -1;
+                    if((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_REJECT))
+                        return -1;
+                    if((msr->txcfg->is_enabled == MODSEC_DETECTION_ONLY) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_REJECT))
+                        return -1;
                 }
 
-                msr->reqbody_length += buflen;
             }
 
             if (APR_BUCKET_IS_EOS(bucket)) {
@@ -259,7 +315,7 @@ apr_status_t read_request_body(modsec_rec *msr, char **error_msg) {
 
     if (msr->txcfg->debuglog_level >= 4) {
         msr_log(msr, 4, "Input filter: Completed receiving request body (length %" APR_SIZE_T_FMT ").",
-            msr->reqbody_length);
+                msr->reqbody_length);
     }
 
     msr->if_status = IF_STATUS_WANTS_TO_RUN;
@@ -391,6 +447,7 @@ static apr_status_t output_filter_init(modsec_rec *msr, ap_filter_t *f,
         if (len > msr->txcfg->of_limit) {
             msr_log(msr, 1, "Output filter: Content-Length (%s) over the limit (%ld).",
                 log_escape_nq(r->pool, (char *)s_content_length), msr->txcfg->of_limit);
+            msr->outbound_error = 1;
             return -2; /* Over the limit. */
         }
     }
@@ -437,6 +494,29 @@ static apr_status_t send_of_brigade(modsec_rec *msr, ap_filter_t *f) {
     return APR_SUCCESS;
 }
 
+static void inject_content_to_of_brigade(modsec_rec *msr, ap_filter_t *f) {
+    apr_bucket *b;
+
+    if (msr->txcfg->content_injection_enabled && msr->stream_output_data != NULL) {
+        apr_bucket *bucket_ci = NULL;
+
+        bucket_ci = apr_bucket_heap_create(msr->stream_output_data,
+                msr->stream_output_length, NULL, f->r->connection->bucket_alloc);
+
+        for (b = APR_BRIGADE_FIRST(msr->of_brigade); b != APR_BRIGADE_SENTINEL(msr->of_brigade); b = APR_BUCKET_NEXT(b))  {
+            if(!APR_BUCKET_IS_METADATA(b))
+                apr_bucket_delete(b);
+        }
+
+        APR_BRIGADE_INSERT_HEAD(msr->of_brigade, bucket_ci);
+
+        if (msr->txcfg->debuglog_level >= 9) {
+            msr_log(msr, 9, "Content Injection: Data reinjected bytes [%"APR_SIZE_T_FMT"]",msr->stream_output_length);
+        }
+
+    }
+}
+
 /**
  *
  */
@@ -445,12 +525,12 @@ static void prepend_content_to_of_brigade(modsec_rec *msr, ap_filter_t *f) {
         apr_bucket *bucket_ci = NULL;
 
         bucket_ci = apr_bucket_heap_create(msr->content_prepend,
-        msr->content_prepend_len, NULL, f->r->connection->bucket_alloc);
-            APR_BRIGADE_INSERT_HEAD(msr->of_brigade, bucket_ci);
+                msr->content_prepend_len, NULL, f->r->connection->bucket_alloc);
+        APR_BRIGADE_INSERT_HEAD(msr->of_brigade, bucket_ci);
 
         if (msr->txcfg->debuglog_level >= 9) {
             msr_log(msr, 9, "Content Injection (b): Added content to top: %s",
-                log_escape_nq_ex(msr->mp, msr->content_prepend, msr->content_prepend_len));
+                    log_escape_nq_ex(msr->mp, msr->content_prepend, msr->content_prepend_len));
         }
     }
 }
@@ -471,20 +551,53 @@ static int flatten_response_body(modsec_rec *msr) {
     msr->resbody_data = apr_palloc(msr->mp, msr->resbody_length + 1);
     if (msr->resbody_data == NULL) {
         msr_log(msr, 1, "Output filter: Response body data memory allocation failed. Asked for: %" APR_SIZE_T_FMT,
-            msr->resbody_length + 1);
+                msr->resbody_length + 1);
         return -1;
     }
 
     rc = apr_brigade_flatten(msr->of_brigade, msr->resbody_data, &msr->resbody_length);
     if (rc != APR_SUCCESS) {
         msr_log(msr, 1, "Output filter: Failed to flatten brigade (%d): %s", rc,
-            get_apr_error(msr->mp, rc));
+                get_apr_error(msr->mp, rc));
         return -1;
     }
 
     msr->resbody_data[msr->resbody_length] = '\0';
     msr->resbody_status = RESBODY_STATUS_READ;
 
+    if (msr->txcfg->stream_outbody_inspection)  {
+
+        char *stream_output_body = NULL;
+
+        if(msr->stream_output_data == NULL)
+            msr->stream_output_data = (char *)malloc(msr->resbody_length+1);
+        else    {
+            stream_output_body = (char *)realloc(msr->stream_output_data, msr->resbody_length+1);
+
+            if(stream_output_body == NULL)  {
+                free(msr->stream_output_data);
+                msr->stream_output_data = NULL;
+                msr_log(msr, 1, "Output filter: Stream Response body data memory allocation failed. Asked for: %" APR_SIZE_T_FMT,
+                    msr->stream_output_length + 1);
+                return -1;
+            }
+
+            msr->stream_output_data = (char *)stream_output_body;
+        }
+
+        msr->stream_output_length = msr->resbody_length;
+
+        if (msr->stream_output_data == NULL) {
+            msr_log(msr, 1, "Output filter: Stream Response body data memory allocation failed. Asked for: %" APR_SIZE_T_FMT,
+                    msr->stream_output_length + 1);
+            return -1;
+        }
+
+        memset(msr->stream_output_data, 0, msr->stream_output_length+1);
+        strncpy(msr->stream_output_data, msr->resbody_data, msr->stream_output_length);
+        msr->stream_output_data[msr->stream_output_length] = '\0';
+    }
+
     return 1;
 }
 
@@ -512,6 +625,22 @@ apr_status_t output_filter(ap_filter_t *f, apr_bucket_brigade *bb_in) {
         msr_log(msr, 9, "Output filter: Receiving output (f %pp, r %pp).", f, f->r);
     }
 
+    /* Put back the Accept-Encoding and TE request headers
+     * if they were removed from the request.
+     */
+    if (msr->txcfg->disable_backend_compression) {
+        char *ae = (char *)apr_table_get(msr->request_headers, "Accept-Encoding");
+        char *te = (char *)apr_table_get(msr->request_headers, "TE");
+
+        if ((ae != NULL)&&(apr_table_get(f->r->headers_in, "Accept-Encoding") == NULL)) {
+            apr_table_add(f->r->headers_in, "Accept-Encoding", ae);
+        }
+
+        if ((te != NULL)&&(apr_table_get(f->r->headers_in, "TE") == NULL)) {
+            apr_table_add(f->r->headers_in, "TE", te);
+        }
+    }
+
     /* Initialise on first invocation */
     if (msr->of_status == OF_STATUS_NOT_STARTED) {
         /* Update our context from the request structure. */
@@ -539,6 +668,7 @@ apr_status_t output_filter(ap_filter_t *f, apr_bucket_brigade *bb_in) {
             }
         }
 
+        msr->outbound_error = 0;
         /* Decide whether to observe the response body. */
         rc = output_filter_init(msr, f, bb_in);
         switch(rc) {
@@ -641,6 +771,7 @@ apr_status_t output_filter(ap_filter_t *f, apr_bucket_brigade *bb_in) {
                  * ready to accept. We need to decide what we want to do
                  * about it.
                  */
+                msr->outbound_error = 1;
                 if (msr->txcfg->of_limit_action == RESPONSE_BODY_LIMIT_ACTION_REJECT) {
                     /* Reject response. */
                     msr_log(msr, 1, "Output filter: Response body too large (over limit of %ld, "
@@ -782,8 +913,7 @@ apr_status_t output_filter(ap_filter_t *f, apr_bucket_brigade *bb_in) {
      * (full-buffering only).
      */
     if ((msr->of_skipping == 0)&&(!msr->of_partial)) {
-        record_time_checkpoint(msr, 3);
-
+        inject_content_to_of_brigade(msr,f);
         prepend_content_to_of_brigade(msr, f);
 
         /* Inject content into response (append & buffering). */
