@@ -1,21 +1,17 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include "modsecurity.h"
 #include "apache2.h"
 #include "http_core.h"
@@ -163,40 +159,6 @@ int apache2_exec(modsec_rec *msr, const char *command, const char **argv, char *
     return 1;
 }
 
-/**
- * Record the current time and store for later.
- */
-void record_time_checkpoint(modsec_rec *msr, int checkpoint_no) {
-    char note[100], note_name[100];
-    apr_time_t now;
-
-    now = apr_time_now();
-    switch(checkpoint_no) {
-        case 1 :
-            msr->time_checkpoint_1 = now;
-            break;
-        case 2 :
-            msr->time_checkpoint_2 = now;
-            break;
-        case 3 :
-            msr->time_checkpoint_3 = now;
-            break;
-        default :
-            msr_log(msr, 1, "Internal Error: Unknown checkpoint: %d", checkpoint_no);
-            return;
-            break;
-    }
-
-    /* Apache-specific stuff. */
-    apr_snprintf(note, 99, "%" APR_TIME_T_FMT, (now - msr->request_time));
-    apr_snprintf(note_name, 99, "mod_security-time%d", checkpoint_no);
-    apr_table_set(msr->r->notes, note_name, note);
-
-    if (msr->txcfg->debuglog_level >= 4) {
-        msr_log(msr, 4, "Time #%d: %s", checkpoint_no, note);
-    }
-}
-
 /**
  * Returns a new string that contains the error
  * message for the given return code.
@@ -230,7 +192,7 @@ char *get_env_var(request_rec *r, char *name) {
  * true, the message will be stripped of any trailing newline and any
  * required bytes will be escaped.
  */
-void internal_log_ex(request_rec *r, directory_config *dcfg, modsec_rec *msr,
+static void internal_log_ex(request_rec *r, directory_config *dcfg, modsec_rec *msr,
     int level, int fixup, const char *text, va_list ap)
 {
     apr_size_t nbytes, nbytes_written;
@@ -317,15 +279,6 @@ void internal_log_ex(request_rec *r, directory_config *dcfg, modsec_rec *msr,
     return;
 }
 
-/**
- * Internal log helper function. Use msr_log instead.
- */
-void internal_log(request_rec *r, directory_config *dcfg, modsec_rec *msr,
-    int level, const char *text, va_list ap)
-{
-    internal_log_ex(r, dcfg, msr, level, 0, text, ap);
-}
-
 /**
  * Logs one message at the given level to the debug log (and to the
  * Apache error log if the message is important enough.
