@@ -1,31 +1,29 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include <limits.h>
 
 #include "http_core.h"
 #include "http_request.h"
-#include "http_connection.h"
 
 #include "modsecurity.h"
 #include "apache2.h"
 #include "http_main.h"
-#include "pdf_protect.h"
+#include "http_connection.h"
+
+#include "apr_optional.h"
+#include "mod_log_config.h"
 
 #include "msc_logging.h"
 #include "msc_util.h"
@@ -33,6 +31,8 @@
 #include "ap_mpm.h"
 #include "scoreboard.h"
 
+#include "apr_version.h"
+
 /* ModSecurity structure */
 
 msc_engine DSOLOCAL *modsecurity = NULL;
@@ -41,11 +41,9 @@ msc_engine DSOLOCAL *modsecurity = NULL;
 
 char DSOLOCAL *chroot_dir = NULL;
 
-unsigned int DSOLOCAL chroot_completed = 0;
-
 char DSOLOCAL *new_server_signature = NULL;
 
-char DSOLOCAL *real_server_signature = NULL;
+static char *real_server_signature = NULL;
 
 char DSOLOCAL *guardianlog_name = NULL;
 
@@ -59,6 +57,8 @@ unsigned long int DSOLOCAL msc_pcre_match_limit_recursion = 0;
 
 unsigned long int DSOLOCAL conn_read_state_limit = 0;
 
+unsigned long int DSOLOCAL conn_write_state_limit = 0;
+
 static int server_limit, thread_limit;
 
 typedef struct {
@@ -68,6 +68,43 @@ typedef struct {
 
 /* -- Miscellaneous functions -- */
 
+/*
+* \brief Print informations from used libraries
+*
+* \param mp Pointer to memory pool
+*/
+static void version(apr_pool_t *mp) {
+    char *pcre_vrs = NULL;
+
+    ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL,
+            "ModSecurity: APR compiled version=\"%s\"; "
+            "loaded version=\"%s\"", APR_VERSION_STRING, apr_version_string());
+
+    if (strstr(apr_version_string(),APR_VERSION_STRING) == NULL)    {
+        ap_log_error(APLOG_MARK, APLOG_WARNING, 0, NULL, "ModSecurity: Loaded APR do not match with compiled!");
+    }
+
+    pcre_vrs = apr_psprintf(mp,"%d.%d", PCRE_MAJOR, PCRE_MINOR);
+
+    ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL,
+            "ModSecurity: PCRE compiled version=\"%s\"; "
+            "loaded version=\"%s\"", pcre_vrs, pcre_version());
+
+    if (strstr(pcre_version(),pcre_vrs) == NULL)    {
+        ap_log_error(APLOG_MARK, APLOG_WARNING, 0, NULL, "ModSecurity: Loaded PCRE do not match with compiled!");
+    }
+
+    /* Lua version function was removed in current 5.1. Need to check in future versions if it's back */
+#if defined(WITH_LUA)
+    ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL,
+            "ModSecurity: LUA compiled version=\"%s\"",LUA_VERSION);
+#endif /* WITH_LUA */
+
+    ap_log_error(APLOG_MARK, APLOG_NOTICE, 0, NULL,
+            "ModSecurity: LIBXML compiled version=\"%s\"",LIBXML_DOTTED_VERSION);
+}
+
+
 /**
  * Intercepts transaction, using the method specified
  * in the structure itself. MUST return an HTTP status code,
@@ -454,10 +491,26 @@ static apr_status_t module_cleanup(void *data) {
     return APR_SUCCESS;
 }
 
+/**
+ * Generate a single variable for use with mod_log_config.
+ */
+static const char *modsec_var_log_handler(request_rec *r, char *name) {
+    modsec_rec *msr = NULL;
+
+    if (name == NULL) return NULL;
+
+    msr = retrieve_tx_context(r);
+    if (msr == NULL) return NULL;
+
+    return construct_single_var(msr, name);
+}
+
 /**
  * Pre-configuration initialisation hook.
  */
 static int hook_pre_config(apr_pool_t *mp, apr_pool_t *mp_log, apr_pool_t *mp_temp) {
+    static APR_OPTIONAL_FN_TYPE(ap_register_log_handler) *log_pfn_register;
+
     /* Initialise ModSecurity engine */
     modsecurity = modsecurity_create(mp, MODSEC_ONLINE);
     if (modsecurity == NULL) {
@@ -466,6 +519,11 @@ static int hook_pre_config(apr_pool_t *mp, apr_pool_t *mp_log, apr_pool_t *mp_te
         return HTTP_INTERNAL_SERVER_ERROR;
     }
 
+    log_pfn_register = APR_RETRIEVE_OPTIONAL_FN(ap_register_log_handler);
+    if (log_pfn_register) {
+        log_pfn_register(mp, "M", modsec_var_log_handler, 0);
+    }
+
     return OK;
 }
 
@@ -534,8 +592,6 @@ static int hook_post_config(apr_pool_t *mp, apr_pool_t *mp_log, apr_pool_t *mp_t
                 exit(1);
             }
 
-            chroot_completed = 1;
-
             ap_log_error(APLOG_MARK, APLOG_NOTICE | APLOG_NOERRNO, 0, s,
                 "ModSecurity: chroot successful, path=%s", chroot_dir);
         } else {
@@ -553,6 +609,8 @@ static int hook_post_config(apr_pool_t *mp, apr_pool_t *mp_log, apr_pool_t *mp_t
         ap_log_error(APLOG_MARK, APLOG_NOTICE | APLOG_NOERRNO, 0, s,
                 "%s configured.", MODSEC_MODULE_NAME_FULL);
 
+        version(mp);
+
         /* If we've changed the server signature make note of the original. */
         if (new_server_signature != NULL) {
             ap_log_error(APLOG_MARK, APLOG_NOTICE | APLOG_NOERRNO, 0, s,
@@ -574,11 +632,11 @@ static void hook_child_init(apr_pool_t *mp, server_rec *s) {
 
 /**
  * Initial request processing, executed immediatelly after
- * Apache receives the request headers.
+ * Apache receives the request headers. This function wil create
+ * a transaction context.
  */
 static int hook_request_early(request_rec *r) {
     modsec_rec *msr = NULL;
-    int rc;
 
     /* This function needs to run only once per transaction
      * (i.e. subrequests and redirects are excluded).
@@ -593,33 +651,22 @@ static int hook_request_early(request_rec *r) {
     msr = create_tx_context(r);
     if (msr == NULL) return DECLINED;
 
+    #if 0
+    /* NOTE This check is not currently needed, but it may be needed in the
+     *      future when we add another early phase.
+     */
+
     /* Are we allowed to continue? */
     if (msr->txcfg->is_enabled == MODSEC_DISABLED) {
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Processing disabled, skipping (hook request_early).");
         }
+
         return DECLINED;
-    }
+    }    
+    #endif
 
-    /* Process phase REQUEST_HEADERS */
-    rc = DECLINED;
-    if (modsecurity_process_phase(msr, PHASE_REQUEST_HEADERS) > 0) {
-        rc = perform_interception(msr);
-    }
-
-    if (    (msr->txcfg->is_enabled != MODSEC_DISABLED)
-         && (msr->txcfg->reqbody_access == 1)
-         && (rc == DECLINED))
-    {
-        /* Check request body limit (non-chunked requests only). */
-        if (msr->request_content_length > msr->txcfg->reqbody_limit) {
-            msr_log(msr, 1, "Request body (Content-Length) is larger than the "
-                         "configured limit (%ld).", msr->txcfg->reqbody_limit);
-            return HTTP_REQUEST_ENTITY_TOO_LARGE;
-        }
-    }
-
-    return rc;
+    return DECLINED;
 }
 
 /**
@@ -654,8 +701,8 @@ static int hook_request_late(request_rec *r) {
         msr_log(msr, 1, "Internal Error: Attempted to process the request body more than once.");
         return DECLINED;
     }
-    msr->phase_request_body_complete = 1;
 
+    msr->phase_request_body_complete = 1;
     msr->remote_user = r->user;
 
     /* Get the second configuration context. */
@@ -675,29 +722,75 @@ static int hook_request_late(request_rec *r) {
 
     init_directory_config(msr->txcfg);
 
-    /* Perform the PDF tests now. */
-    rc = pdfp_check(msr);
-    if (rc > 0) {
-        /* The PDF protection module has decided it needs to
-         * redirect the current transaction. So we let it do that.
-         */
-        return rc;
-    }
-
     if (msr->txcfg->is_enabled == 0) {
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Processing disabled, skipping (hook request_late).");
         }
+
         return DECLINED;
     }
 
+    /* Phase 1 */
+    if (msr->txcfg->debuglog_level >= 4) {
+        msr_log(msr, 4, "First phase starting (dcfg %pp).", msr->dcfg2);
+    }
+
+    /* Process phase REQUEST_HEADERS */
+    if (modsecurity_process_phase(msr, PHASE_REQUEST_HEADERS) > 0) {
+        /* There was a match; see if we need to intercept. */
+        rc = perform_interception(msr);
+        if (rc != DECLINED) {
+            /* Intercepted */
+            return rc;
+        }
+    }
+
+    /* The rule engine could have been disabled in phase 1. */
+    if (msr->txcfg->is_enabled == MODSEC_DISABLED) {
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Skipping phase 2 as the rule engine was disabled by a rule in phase 1.");
+        }
+
+        return DECLINED;
+    }
+
+    /* Phase 2 */
     if (msr->txcfg->debuglog_level >= 4) {
         msr_log(msr, 4, "Second phase starting (dcfg %pp).", msr->dcfg2);
     }
 
-    /* Figure out whether or not to extract multipart files. */
+    /* Check that the request body is not too long, but only
+     * if configuration allows for request body access.
+     */
+    msr->inbound_error = 0;
+    if (msr->txcfg->reqbody_access == 1) {
+        /* Check request body limit (non-chunked requests only). */
+        if (msr->request_content_length > msr->txcfg->reqbody_limit) {
+
+            if((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_REJECT)) {
+                msr->inbound_error = 1;
+                msr_log(msr, 1, "Request body (Content-Length) is larger than the "
+                        "configured limit (%ld). Deny with status (%d)", msr->txcfg->reqbody_limit, HTTP_REQUEST_ENTITY_TOO_LARGE);
+                return HTTP_REQUEST_ENTITY_TOO_LARGE;
+            } else if ((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)){
+                msr->inbound_error = 1;
+                msr_log(msr, 1, "Request body (Content-Length) is larger than the "
+                        "configured limit (%ld).", msr->txcfg->reqbody_limit);
+            } else if ((msr->txcfg->is_enabled == MODSEC_DETECTION_ONLY) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_PARTIAL)){
+                msr_log(msr, 1, "Request body (Content-Length) is larger than the "
+                        "configured limit (%ld).", msr->txcfg->reqbody_limit);
+                msr->inbound_error = 1;
+            } else  {
+                msr_log(msr, 1, "Request body (Content-Length) is larger than the "
+                        "configured limit (%ld).", msr->txcfg->reqbody_limit);
+                msr->inbound_error = 1;
+            }
+        }
+    }
+
+    /* Figure out whether to extract multipart files. */
     if ((msr->txcfg->upload_keep_files != KEEP_FILES_OFF) /* user might want to keep them */
-        || (msr->txcfg->upload_validates_files)) /* user might want to validate them */
+            || (msr->txcfg->upload_validates_files)) /* user might want to validate them */
     {
         msr->upload_extract_files = 1;
         msr->upload_remove_files = 1;
@@ -720,11 +813,25 @@ static int hook_request_late(request_rec *r) {
                 return HTTP_REQUEST_TIME_OUT;
                 break;
             case -5 : /* Request body limit reached. */
+                msr->inbound_error = 1;
+                if((msr->txcfg->is_enabled == MODSEC_ENABLED) && (msr->txcfg->if_limit_action == REQUEST_BODY_LIMIT_ACTION_REJECT))    {
+                    r->connection->keepalive = AP_CONN_CLOSE;
+                    if (my_error_msg != NULL) {
+                        msr_log(msr, 1, "%s. Deny with code (%d)", my_error_msg, HTTP_REQUEST_ENTITY_TOO_LARGE);
+                    }
+                    return HTTP_REQUEST_ENTITY_TOO_LARGE;
+                } else  {
+                    if (my_error_msg != NULL) {
+                        msr_log(msr, 1, "%s", my_error_msg);
+                    }
+                }
+                break;
+            case -6 : /* EOF when reading request body. */
                 if (my_error_msg != NULL) {
-                    msr_log(msr, 1, "%s", my_error_msg);
+                    msr_log(msr, 4, "%s", my_error_msg);
                 }
                 r->connection->keepalive = AP_CONN_CLOSE;
-                return HTTP_REQUEST_ENTITY_TOO_LARGE;
+                return HTTP_BAD_REQUEST;
                 break;
             default :
                 /* allow through */
@@ -737,21 +844,35 @@ static int hook_request_late(request_rec *r) {
 
     /* Update the request headers. They might have changed after
      * the body was read (trailers).
-     */
-    /* NOTE We still need to keep a copy of the original headers
-     *      to log in the audit log.
+     *
+     * TODO We might still want to hold onto the original headers
+     *      so that we can log them. Keeping them is probably not
+     *      going to increase our memory requirements (because all
+     *      headers are allocated from the request memory pool
+     *      anyway).
      */
     msr->request_headers = apr_table_copy(msr->mp, r->headers_in);
 
     /* Process phase REQUEST_BODY */
-    record_time_checkpoint(msr, 1);
-
     rc = DECLINED;
     if (modsecurity_process_phase(msr, PHASE_REQUEST_BODY) > 0) {
         rc = perform_interception(msr);
     }
 
-    record_time_checkpoint(msr, 2);
+    if(msr->txcfg->stream_inbody_inspection && msr->msc_reqbody_read)    {
+        const char *clen = NULL;
+        clen = apr_psprintf(msr->mp,"%"APR_SIZE_T_FMT,msr->stream_input_length);
+        if(clen)
+            apr_table_setn(r->headers_in, "Content-Length",clen);
+    }
+
+    /* Remove the compression ability indications the client set,
+     * but only if we need to disable backend compression.
+     */
+    if (msr->txcfg->disable_backend_compression) {
+        apr_table_unset(r->headers_in, "Accept-Encoding");
+        apr_table_unset(r->headers_in, "TE");
+    }
 
     return rc;
 }
@@ -759,8 +880,13 @@ static int hook_request_late(request_rec *r) {
 /**
  * Invoked every time Apache has something to write to the error log.
  */
+#if AP_SERVER_MAJORVERSION_NUMBER > 1 && AP_SERVER_MINORVERSION_NUMBER > 2
+static void hook_error_log(const char *file, int line, int module_index, int level, apr_status_t status,
+        const server_rec *s, const request_rec *r, apr_pool_t *mp, const char *fmt)
+#else
 static void hook_error_log(const char *file, int line, int level, apr_status_t status,
-    const server_rec *s, const request_rec *r, apr_pool_t *mp, const char *fmt)
+        const server_rec *s, const request_rec *r, apr_pool_t *mp, const char *fmt)
+#endif
 {
     modsec_rec *msr = NULL;
     error_message *em = NULL;
@@ -977,13 +1103,6 @@ static void hook_insert_filter(request_rec *r) {
         return;
     }
 
-    /* We always add the PDF XSS protection filter. */
-    if (msr->txcfg->debuglog_level >= 4) {
-        msr_log(msr, 4, "Hook insert_filter: Adding PDF XSS protection output filter (r %pp).", r);
-    }
-
-    ap_add_output_filter("PDFP_OUT", msr, r, r->connection);
-
     /* Only proceed to add the second filter if the engine is enabled. */
     if (msr->txcfg->is_enabled == 0) {
         if (msr->txcfg->debuglog_level >= 4) {
@@ -1007,7 +1126,6 @@ static void hook_insert_filter(request_rec *r) {
 }
 
 // TODO: Holding off on this for now (needs more testing)
-#if 0
 /**
  * Invoked whenever Apache starts processing an error. A chance
  * to insert ourselves into the output filter chain.
@@ -1049,7 +1167,6 @@ static void hook_insert_error_filter(request_rec *r) {
         }
     }
 }
-#endif
 
 #if (!defined(NO_MODSEC_API))
 /**
@@ -1085,21 +1202,32 @@ static int hook_connection_early(conn_rec *conn)
 {
     sb_handle *sb = conn->sbh;
     int i, j;
-    unsigned long int ip_count = 0;
+    unsigned long int ip_count = 0, ip_count_w = 0;
     worker_score *ws_record = NULL;
+#if AP_SERVER_MINORVERSION_NUMBER > 1 && AP_SERVER_MINORVERSION_NUMBER > 2
+    ap_sb_handle_t *sbh = NULL;
+#endif
 
-    if(sb != NULL && conn_read_state_limit > 0)   {
+    if(sb != NULL && (conn_read_state_limit > 0 || conn_write_state_limit > 0))   {
 
         ws_record = &ap_scoreboard_image->servers[sb->child_num][sb->thread_num];
         if(ws_record == NULL)
             return DECLINED;
 
         apr_cpystrn(ws_record->client, conn->remote_ip, sizeof(ws_record->client));
-
         for (i = 0; i < server_limit; ++i) {
             for (j = 0; j < thread_limit; ++j) {
 
+#if AP_SERVER_MINORVERSION_NUMBER > 1 && AP_SERVER_MINORVERSION_NUMBER > 2
+                sbh = conn->sbh;
+                if (sbh == NULL)        {
+                    return DECLINED;
+                }
+
+                ws_record = ap_get_scoreboard_worker(sbh);
+#else
                 ws_record = ap_get_scoreboard_worker(i, j);
+#endif
 
                 if(ws_record == NULL)
                     return DECLINED;
@@ -1109,15 +1237,22 @@ static int hook_connection_early(conn_rec *conn)
                         if (strcmp(conn->remote_ip, ws_record->client) == 0)
                             ip_count++;
                         break;
+                    case SERVER_BUSY_WRITE:
+                        if (strcmp(conn->remote_ip, ws_record->client) == 0)
+                            ip_count_w++;
+                        break;
                     default:
                         break;
                 }
             }
         }
 
-        if (ip_count > conn_read_state_limit) {
+        if ((conn_read_state_limit > 0) && (ip_count > conn_read_state_limit)) {
             ap_log_error(APLOG_MARK, APLOG_WARNING, 0, NULL, "ModSecurity: Access denied with code 400. Too many threads [%ld] of %ld allowed in READ state from %s - Possible DoS Consumption Attack [Rejected]", ip_count,conn_read_state_limit,conn->remote_ip);
             return OK;
+        } else if ((conn_write_state_limit > 0) && (ip_count_w > conn_write_state_limit)) {
+            ap_log_error(APLOG_MARK, APLOG_WARNING, 0, NULL, "ModSecurity: Access denied with code 400. Too many threads [%ld] of %ld allowed in WRITE state from %s - Possible DoS Consumption Attack [Rejected]", ip_count_w,conn_write_state_limit,conn->remote_ip);
+            return OK;
         } else {
             return DECLINED;
         }
@@ -1126,14 +1261,15 @@ static int hook_connection_early(conn_rec *conn)
     return DECLINED;
 }
 
+
 /**
  * This function is exported for other Apache modules to
  * register new variables.
  */
 static void modsec_register_variable(const char *name, unsigned int type,
-                                     unsigned int argc_min, unsigned int argc_max,
-                                     void *fn_validate, void *fn_generate,
-                                     unsigned int is_cacheable, unsigned int availability) {
+        unsigned int argc_min, unsigned int argc_max,
+        void *fn_validate, void *fn_generate,
+        unsigned int is_cacheable, unsigned int availability) {
     if (modsecurity != NULL) {
         msre_engine_variable_register(modsecurity->msre, name, type, argc_min, argc_max, (fn_var_validate_t)fn_validate, (fn_var_generate_t)fn_generate, is_cacheable, availability);
     }
@@ -1141,23 +1277,44 @@ static void modsec_register_variable(const char *name, unsigned int type,
         fprintf(stderr,"modsecurity is NULL\n");
     }
 }
+
+/**
+ * This function is exported for other Apache modules to
+ * register new request body processors.
+ */
+static void modsec_register_reqbody_processor(const char *name,
+        void *fn_init,
+        void *fn_process,
+        void *fn_complete)
+{
+    if (modsecurity != NULL) {
+
+        msre_engine_reqbody_processor_register(modsecurity->msre, name,
+                (fn_reqbody_processor_init_t)fn_init,
+                (fn_reqbody_processor_init_t)fn_process,
+                (fn_reqbody_processor_init_t)fn_complete);
+    }
+}
+
 #endif
 
 /**
  * Registers module hooks with Apache.
  */
 static void register_hooks(apr_pool_t *mp) {
-    static const char *postconfig_beforeme_list[] = {
+    static const char *const postconfig_beforeme_list[] = {
         "mod_unique_id.c",
         "mod_ssl.c",
         NULL
     };
-    static const char *postconfig_afterme_list[] = {
+
+    static const char *const postconfig_afterme_list[] = {
         "mod_fcgid.c",
         "mod_cgid.c",
         NULL
     };
-    static const char *postread_beforeme_list[] = {
+
+    static const char *const postread_beforeme_list[] = {
         "mod_rpaf.c",
         "mod_rpaf-2.0.c",
         "mod_extract_forwarded2.c",
@@ -1168,11 +1325,20 @@ static void register_hooks(apr_pool_t *mp) {
         "mod_unique_id.c",
         NULL
     };
-    static const char *postread_afterme_list[] = {
+
+    static const char *const postread_afterme_list[] = {
         "mod_log_forensic.c",
         NULL
     };
 
+    static const char *const transaction_afterme_list[] = {
+        "mod_log_config.c",
+        NULL
+    };
+
+    /* Add the MODSEC_2.x compatibility defines */
+    *(char **)apr_array_push(ap_server_config_defines) = apr_pstrdup(mp, "MODSEC_2.5");
+
     /* Add the MODSEC_a.b define */
     *(char **)apr_array_push(ap_server_config_defines) = apr_psprintf(mp, "MODSEC_%s.%s", MODSEC_VERSION_MAJOR, MODSEC_VERSION_MINOR);
 
@@ -1181,6 +1347,7 @@ static void register_hooks(apr_pool_t *mp) {
     APR_REGISTER_OPTIONAL_FN(modsec_register_tfn);
     APR_REGISTER_OPTIONAL_FN(modsec_register_operator);
     APR_REGISTER_OPTIONAL_FN(modsec_register_variable);
+    APR_REGISTER_OPTIONAL_FN(modsec_register_reqbody_processor);
 #endif
 
     /* For connection level hook */
@@ -1208,13 +1375,12 @@ static void register_hooks(apr_pool_t *mp) {
 
     /* Logging */
     ap_hook_error_log(hook_error_log, NULL, NULL, APR_HOOK_MIDDLE);
-    ap_hook_log_transaction(hook_log_transaction, NULL, NULL, APR_HOOK_MIDDLE);
+    ap_hook_log_transaction(hook_log_transaction, NULL, transaction_afterme_list, APR_HOOK_MIDDLE);
 
     /* Filter hooks */
     ap_hook_insert_filter(hook_insert_filter, NULL, NULL, APR_HOOK_FIRST);
-#if 0
+
     ap_hook_insert_error_filter(hook_insert_error_filter, NULL, NULL, APR_HOOK_FIRST);
-#endif
 
     ap_register_input_filter("MODSECURITY_IN", input_filter,
         NULL, AP_FTYPE_CONTENT_SET);
@@ -1228,11 +1394,10 @@ static void register_hooks(apr_pool_t *mp) {
      *   mod_deflate = -1
      *   mod_headers =  0
      */
+
     ap_register_output_filter("MODSECURITY_OUT", output_filter,
         NULL, AP_FTYPE_CONTENT_SET - 3);
 
-    ap_register_output_filter("PDFP_OUT", pdfp_output_filter,
-        NULL, AP_FTYPE_CONTENT_SET);
 }
 
 /* Defined in apache2_config.c */
