@@ -1,21 +1,17 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include <stdlib.h>
 
 #include "apr_global_mutex.h"
@@ -24,6 +20,7 @@
 #include "msc_parsers.h"
 #include "msc_util.h"
 #include "msc_xml.h"
+#include "apr_version.h"
 
 /**
  * Format an alert message.
@@ -80,6 +77,7 @@ static const char *phase_name(int phase) {
             return "LOGGING";
             break;
     }
+    
     return "INVALID";
 }
 #endif
@@ -102,6 +100,7 @@ msc_engine *modsecurity_create(apr_pool_t *mp, int processing_mode) {
     msre_engine_register_default_operators(msce->msre);
     msre_engine_register_default_tfns(msce->msre);
     msre_engine_register_default_actions(msce->msre);
+    // TODO: msre_engine_register_default_reqbody_processors(msce->msre);
 
     return msce;
 }
@@ -122,26 +121,35 @@ int modsecurity_init(msc_engine *msce, apr_pool_t *mp) {
         return -1;
     }
 
-    #ifdef __SET_MUTEX_PERMS
+#ifdef __SET_MUTEX_PERMS
+#if AP_SERVER_MAJORVERSION_NUMBER > 1 && AP_SERVER_MINORVERSION_NUMBER > 2
+    rc = ap_unixd_set_global_mutex_perms(msce->auditlog_lock);
+#else
     rc = unixd_set_global_mutex_perms(msce->auditlog_lock);
+#endif
     if (rc != APR_SUCCESS) {
         // ap_log_error(APLOG_MARK, APLOG_ERR, rc, s, "mod_security: Could not set permissions on modsec_auditlog_lock; check User and Group directives");
         // return HTTP_INTERNAL_SERVER_ERROR;
         return -1;
     }
-    #endif
+#endif /* SET_MUTEX_PERMS */
 
     rc = apr_global_mutex_create(&msce->geo_lock, NULL, APR_LOCK_DEFAULT, mp);
     if (rc != APR_SUCCESS) {
         return -1;
     }
 
-    #ifdef __SET_MUTEX_PERMS
+#ifdef __SET_MUTEX_PERMS
+#if AP_SERVER_MAJORVERSION_NUMBER > 1 && AP_SERVER_MINORVERSION_NUMBER > 2
+    rc = ap_unixd_set_global_mutex_perms(msce->geo_lock);
+#else
     rc = unixd_set_global_mutex_perms(msce->geo_lock);
+#endif
     if (rc != APR_SUCCESS) {
         return -1;
     }
-    #endif
+#endif /* SET_MUTEX_PERMS */
+
 
     return 1;
 }
@@ -179,19 +187,13 @@ void modsecurity_shutdown(msc_engine *msce) {
 /**
  *
  */
-static apr_status_t modsecurity_tx_cleanup(void *data) {
-    modsec_rec *msr = (modsec_rec *)data;
+static void modsecurity_persist_data(modsec_rec *msr) {
     const apr_array_header_t *arr;
     apr_table_entry_t *te;
-    int collect_garbage = 0;
+    apr_time_t time_before, time_after;
     int i;
-    char *my_error_msg = NULL;
 
-    if (msr == NULL) return APR_SUCCESS;
-
-    if (rand() < RAND_MAX/100) {
-        collect_garbage = 1;
-    }
+    time_before = apr_time_now();
 
     /* Collections, store & remove stale. */
     arr = apr_table_elts(msr->collections);
@@ -203,11 +205,42 @@ static apr_status_t modsecurity_tx_cleanup(void *data) {
         if (apr_table_get(msr->collections_dirty, te[i].key)) {
             collection_store(msr, col);
         }
+    }
 
-        if (collect_garbage) {
+    time_after = apr_time_now();
+    
+    msr->time_storage_write += time_after - time_before;
+    
+    if (msr->txcfg->debuglog_level >= 4) {
+        msr_log(msr, 4, "Recording persistent data took %" APR_TIME_T_FMT
+            " microseconds.", msr->time_gc);
+    }   
+    
+    /* Remove stale collections. */
+    if (rand() < RAND_MAX/100) {
+        arr = apr_table_elts(msr->collections);
+        te = (apr_table_entry_t *)arr->elts;
+        for (i = 0; i < arr->nelts; i++) {
             collections_remove_stale(msr, te[i].key);
         }
+        
+        msr->time_gc = apr_time_now() - time_after;
+        
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Garbage collection took %" APR_TIME_T_FMT
+                " microseconds.", msr->time_gc);
+        }   
     }
+}
+
+/**
+ *
+ */
+static apr_status_t modsecurity_tx_cleanup(void *data) {
+    modsec_rec *msr = (modsec_rec *)data;
+    char *my_error_msg = NULL;
+    
+    if (msr == NULL) return APR_SUCCESS;    
 
     /* Multipart processor cleanup. */
     if (msr->mpd != NULL) multipart_cleanup(msr);
@@ -315,19 +348,30 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
             msr_log(msr, 1, "Initialisation: Error occurred while parsing QUERY_STRING arguments.");
             return -1;
         }
+
+        if (invalid_count) {
+            msr->urlencoded_error = 1;
+        }
     }
 
-    msr->arguments_to_sanitise = apr_table_make(msr->mp, 16);
-    if (msr->arguments_to_sanitise == NULL) return -1;
-    msr->request_headers_to_sanitise = apr_table_make(msr->mp, 16);
-    if (msr->request_headers_to_sanitise == NULL) return -1;
-    msr->response_headers_to_sanitise = apr_table_make(msr->mp, 16);
-    if (msr->response_headers_to_sanitise == NULL) return -1;
+    msr->arguments_to_sanitize = apr_table_make(msr->mp, 16);
+    if (msr->arguments_to_sanitize == NULL) return -1;
+    msr->request_headers_to_sanitize = apr_table_make(msr->mp, 16);
+    if (msr->request_headers_to_sanitize == NULL) return -1;
+    msr->response_headers_to_sanitize = apr_table_make(msr->mp, 16);
+    if (msr->response_headers_to_sanitize == NULL) return -1;
+    msr->pattern_to_sanitize = apr_table_make(msr->mp, 32);
+    if (msr->pattern_to_sanitize == NULL) return -1;
 
     /* Initialise cookies */
     msr->request_cookies = apr_table_make(msr->mp, 16);
     if (msr->request_cookies == NULL) return -1;
 
+    /* Initialize matched vars */
+    msr->matched_vars = apr_table_make(msr->mp, 8);
+    if (msr->matched_vars == NULL) return -1;
+    apr_table_clear(msr->matched_vars);
+
     /* Locate the cookie headers and parse them */
     arr = apr_table_elts(msr->request_headers);
     te = (apr_table_entry_t *)arr->elts;
@@ -370,6 +414,9 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     msr->removed_rules = apr_array_make(msr->mp, 16, sizeof(char *));
     if (msr->removed_rules == NULL) return -1;
 
+    msr->removed_rules_tag = apr_array_make(msr->mp, 16, sizeof(char *));
+    if (msr->removed_rules_tag == NULL) return -1;
+
     return 1;
 }
 
@@ -398,6 +445,7 @@ static int is_response_status_relevant(modsec_rec *msr, int status) {
     if (rc == PCRE_ERROR_NOMATCH) return 0;
 
     msr_log(msr, 1, "Regex processing failed (rc %d): %s", rc, my_error_msg);
+    
     return 0;
 }
 
@@ -405,78 +453,136 @@ static int is_response_status_relevant(modsec_rec *msr, int status) {
  *
  */
 static apr_status_t modsecurity_process_phase_request_headers(modsec_rec *msr) {
-    msr_log(msr, 4, "Starting phase REQUEST_HEADERS.");
+    apr_time_t time_before;
+    apr_status_t rc = 0;
+    
+    if (msr->txcfg->debuglog_level >= 4) {
+        msr_log(msr, 4, "Starting phase REQUEST_HEADERS.");
+    }
+    
+    time_before = apr_time_now();
 
     if (msr->txcfg->ruleset != NULL) {
-        return msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
+        rc = msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
     }
+    
+    msr->time_phase1 = apr_time_now() - time_before;
 
-    return 0;
+    return rc;
 }
 
 /**
  *
  */
 static apr_status_t modsecurity_process_phase_request_body(modsec_rec *msr) {
+    apr_time_t time_before;
+    apr_status_t rc = 0;
+
+    
     if ((msr->allow_scope == ACTION_ALLOW_REQUEST)||(msr->allow_scope == ACTION_ALLOW)) {
-        msr_log(msr, 4, "Skipping phase REQUEST_BODY (allow used).");
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Skipping phase REQUEST_BODY (allow used).");
+        }
+        
         return 0;
     } else {
-        msr_log(msr, 4, "Starting phase REQUEST_BODY.");
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Starting phase REQUEST_BODY.");
+        }
     }
+    
+    time_before = apr_time_now();
 
     if (msr->txcfg->ruleset != NULL) {
-        return msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
+        rc = msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
     }
+    
+    msr->time_phase2 = apr_time_now() - time_before;
 
-    return 0;
+    return rc;
 }
 
 /**
  *
  */
 static apr_status_t modsecurity_process_phase_response_headers(modsec_rec *msr) {
+    apr_time_t time_before;
+    apr_status_t rc = 0;
+    
     if (msr->allow_scope == ACTION_ALLOW) {
-        msr_log(msr, 4, "Skipping phase RESPONSE_HEADERS (allow used).");
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Skipping phase RESPONSE_HEADERS (allow used).");
+        }
+        
         return 0;
     } else {
-        msr_log(msr, 4, "Starting phase RESPONSE_HEADERS.");
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Starting phase RESPONSE_HEADERS.");
+        }
     }
+    
+    time_before = apr_time_now();
 
     if (msr->txcfg->ruleset != NULL) {
-        return msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
+        rc = msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
     }
+    
+    msr->time_phase3 = apr_time_now() - time_before;
 
-    return 0;
+    return rc;
 }
 
 /**
  *
  */
 static apr_status_t modsecurity_process_phase_response_body(modsec_rec *msr) {
+    apr_time_t time_before;
+    apr_status_t rc = 0;
+    
     if (msr->allow_scope == ACTION_ALLOW) {
-        msr_log(msr, 4, "Skipping phase RESPONSE_BODY (allow used).");
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Skipping phase RESPONSE_BODY (allow used).");
+        }
+        
         return 0;
     } else {
-        msr_log(msr, 4, "Starting phase RESPONSE_BODY.");
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Starting phase RESPONSE_BODY.");
+        }
     }
+    
+    time_before = apr_time_now();
 
     if (msr->txcfg->ruleset != NULL) {
-        return msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
+        rc = msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
     }
+    
+    msr->time_phase4 = apr_time_now() - time_before;
 
-    return 0;
+
+    return rc;
 }
 
 /**
  *
  */
 static apr_status_t modsecurity_process_phase_logging(modsec_rec *msr) {
-    msr_log(msr, 4, "Starting phase LOGGING.");
+    apr_time_t time_before, time_after;
+    
+    if (msr->txcfg->debuglog_level >= 4) {
+        msr_log(msr, 4, "Starting phase LOGGING.");
+    }
+    
+    time_before = apr_time_now();
 
     if (msr->txcfg->ruleset != NULL) {
         msre_ruleset_process_phase(msr->txcfg->ruleset, msr);
     }
+    
+    modsecurity_persist_data(msr);
+    
+    time_after = apr_time_now();
+    msr->time_phase5 = time_after - time_before;
 
     /* Is this request relevant for logging purposes? */
     if (msr->is_relevant == 0) {
@@ -503,13 +609,19 @@ static apr_status_t modsecurity_process_phase_logging(modsec_rec *msr) {
     /* Are we configured for audit logging? */
     switch(msr->txcfg->auditlog_flag) {
         case AUDITLOG_OFF :
-            msr_log(msr, 4, "Audit log: Not configured to run for this request.");
+            if (msr->txcfg->debuglog_level >= 4) {
+                msr_log(msr, 4, "Audit log: Not configured to run for this request.");
+            }
+            
             return DECLINED;
             break;
 
         case AUDITLOG_RELEVANT :
             if (msr->is_relevant == 0) {
-                msr_log(msr, 4, "Audit log: Ignoring a non-relevant request.");
+                if (msr->txcfg->debuglog_level >= 4) {
+                    msr_log(msr, 4, "Audit log: Ignoring a non-relevant request.");
+                }
+                
                 return DECLINED;
             }
             break;
@@ -524,9 +636,13 @@ static apr_status_t modsecurity_process_phase_logging(modsec_rec *msr) {
     }
 
     /* Invoke the Audit logger */
-    msr_log(msr, 4, "Audit log: Logging this transaction.");
+    if (msr->txcfg->debuglog_level >= 4) {
+        msr_log(msr, 4, "Audit log: Logging this transaction.");
+    }
 
     sec_audit_logger(msr);
+    
+    msr->time_logging = apr_time_now() - time_after;    
 
     return 0;
 }
@@ -539,14 +655,20 @@ static apr_status_t modsecurity_process_phase_logging(modsec_rec *msr) {
 apr_status_t modsecurity_process_phase(modsec_rec *msr, unsigned int phase) {
     /* Check if we should run. */
     if ((msr->was_intercepted)&&(phase != PHASE_LOGGING)) {
-        msr_log(msr, 4, "Skipping phase %d as request was already intercepted.", phase);
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Skipping phase %d as request was already intercepted.", phase);
+        }
+        
         return 0;
     }
 
     /* Do not process the same phase twice. */
     if (msr->phase >= phase) {
-        msr_log(msr, 4, "Skipping phase %d because it was previously run (at %d now).",
-            phase, msr->phase);
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Skipping phase %d because it was previously run (at %d now).",
+                phase, msr->phase);
+        }
+        
         return 0;
     }
 
@@ -586,12 +708,15 @@ apr_status_t modsecurity_process_phase(modsec_rec *msr, unsigned int phase) {
                     rec = (msre_cache_rec *)ctelts[ri].val;
                     if (rec->changed) {
                         if (msr->txcfg->debuglog_level >= 9) {
-                            msr_log(msr, 9, "CACHE: %5d) hits=%d key=%pp %x;%s=\"%s\" (%pp - %pp)", cn, rec->hits, key, rec->num, rec->path, log_escape_nq_ex(mp, rec->val, rec->val_len), rec->val, rec->val + rec->val_len);
+                            msr_log(msr, 9, "CACHE: %5d) hits=%d key=%pp %x;%s=\"%s\" (%pp - %pp)",
+                                cn, rec->hits, key, rec->num, rec->path, log_escape_nq_ex(mp, rec->val, rec->val_len),
+                                rec->val, rec->val + rec->val_len);
                         }
                     }
                     else {
                         if (msr->txcfg->debuglog_level >= 9) {
-                            msr_log(msr, 9, "CACHE: %5d) hits=%d key=%pp %x;%s=<no change>", cn, rec->hits, key, rec->num, rec->path);
+                            msr_log(msr, 9, "CACHE: %5d) hits=%d key=%pp %x;%s=<no change>",
+                                cn, rec->hits, key, rec->num, rec->path);
                         }
                     }
                 }
@@ -601,7 +726,9 @@ apr_status_t modsecurity_process_phase(modsec_rec *msr, unsigned int phase) {
                 apr_hash_set(msr->tcache, key, klen, NULL);
             }
 
-            msr_log(msr, 9, "Cleared transformation cache for phase %d", msr->phase);
+            if (msr->txcfg->debuglog_level >= 9) {
+                msr_log(msr, 9, "Cleared transformation cache for phase %d", msr->phase);
+            }
         }
 
         msr->tcache_items = 0;
