@@ -1,26 +1,23 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #ifndef _MODSECURITY_H_
 #define _MODSECURITY_H_
 
 #include <stdio.h>
 #include <stdlib.h>
+
 #include <limits.h>
 
 typedef struct rule_exception rule_exception;
@@ -31,6 +28,7 @@ typedef struct msc_engine msc_engine;
 typedef struct msc_data_chunk msc_data_chunk;
 typedef struct msc_arg msc_arg;
 typedef struct msc_string msc_string;
+typedef struct msc_parm msc_parm;
 
 #include "msc_release.h"
 #include "msc_logging.h"
@@ -39,6 +37,7 @@ typedef struct msc_string msc_string;
 #include "msc_util.h"
 #include "msc_xml.h"
 #include "msc_geo.h"
+#include "msc_gsb.h"
 #include "re.h"
 
 #include "ap_config.h"
@@ -93,6 +92,9 @@ typedef struct msc_string msc_string;
 #define REQUEST_BODY_FORCEBUF_OFF               0
 #define REQUEST_BODY_FORCEBUF_ON                1
 
+#define REQUEST_BODY_LIMIT_ACTION_REJECT       0
+#define REQUEST_BODY_LIMIT_ACTION_PARTIAL      1
+
 #define SECACTION_TARGETS                       "REMOTE_ADDR"
 #define SECACTION_ARGS                          "@unconditionalMatch"
 
@@ -120,7 +122,6 @@ typedef struct msc_string msc_string;
 #define FATAL_ERROR "ModSecurity: Fatal error (memory allocation or unexpected internal error)!"
 
 extern DSOLOCAL char *new_server_signature;
-extern DSOLOCAL char *real_server_signature;
 extern DSOLOCAL char *chroot_dir;
 
 extern module AP_MODULE_DECLARE_DATA security2_module;
@@ -133,6 +134,8 @@ extern DSOLOCAL unsigned long int msc_pcre_match_limit_recursion;
 
 extern DSOLOCAL unsigned long int conn_read_state_limit;
 
+extern DSOLOCAL unsigned long int conn_write_state_limit;
+
 #define RESBODY_STATUS_NOT_READ         0   /* we were not configured to read the body */
 #define RESBODY_STATUS_ERROR            1   /* error occured while we were reading the body */
 #define RESBODY_STATUS_PARTIAL          2   /* partial body content available in the brigade */
@@ -180,6 +183,7 @@ extern DSOLOCAL unsigned long int conn_read_state_limit;
 #define RULE_EXCEPTION_IMPORT_MSG       2
 #define RULE_EXCEPTION_REMOVE_ID        3
 #define RULE_EXCEPTION_REMOVE_MSG       4
+#define RULE_EXCEPTION_REMOVE_TAG       5
 
 #define NBSP                            160
 
@@ -225,6 +229,11 @@ struct modsec_rec {
     char                *resbody_data;
     unsigned int         resbody_contains_html;
 
+    apr_size_t           stream_input_length;
+    char                *stream_input_data;
+    apr_size_t           stream_output_length;
+    char                *stream_output_data;
+
     apr_array_header_t  *error_messages;
     apr_array_header_t  *alerts;
 
@@ -259,10 +268,15 @@ struct modsec_rec {
     const char          *request_content_type;
 
     apr_table_t         *arguments;
-    apr_table_t         *arguments_to_sanitise;
-    apr_table_t         *request_headers_to_sanitise;
-    apr_table_t         *response_headers_to_sanitise;
+    apr_table_t         *arguments_to_sanitize;
+    apr_table_t         *request_headers_to_sanitize;
+    apr_table_t         *response_headers_to_sanitize;
     apr_table_t         *request_cookies;
+    apr_table_t         *pattern_to_sanitize;
+
+    unsigned int        urlencoded_error;
+    unsigned int        inbound_error;
+    unsigned int        outbound_error;
 
     unsigned int         is_relevant;
 
@@ -322,10 +336,16 @@ struct modsec_rec {
 
     /* performance measurement */
     apr_time_t           request_time;
-    apr_time_t           time_checkpoint_1;
-    apr_time_t           time_checkpoint_2;
-    apr_time_t           time_checkpoint_3;
-
+    apr_time_t		 time_phase1;
+    apr_time_t		 time_phase2;
+    apr_time_t		 time_phase3;
+    apr_time_t		 time_phase4;
+    apr_time_t		 time_phase5;
+    apr_time_t		 time_storage_read;
+    apr_time_t		 time_storage_write;
+    apr_time_t		 time_logging;
+    apr_time_t		 time_gc;
+    
     apr_array_header_t  *matched_rules;
     msc_string          *matched_var;
     int                  highest_severity;
@@ -355,6 +375,7 @@ struct modsec_rec {
 
     /* removed rules */
     apr_array_header_t  *removed_rules;
+    apr_array_header_t  *removed_rules_tag;
 
     /* When "allow" is executed the variable below is
      * updated to contain the scope of the allow action. Set
@@ -363,6 +384,12 @@ struct modsec_rec {
      * are to allow phases 1-2 only.
      */
     unsigned int         allow_scope;
+
+    /* matched vars */
+    apr_table_t         *matched_vars;
+
+    /* Generic request body processor context to be used by custom parsers. */
+    void                *reqbody_processor_ctx;
 };
 
 struct directory_config {
@@ -372,6 +399,7 @@ struct directory_config {
 
     int                  is_enabled;
     int                  reqbody_access;
+    int                  reqintercept_oe;
     int                  reqbody_buffering;
     long int             reqbody_inmemory_limit;
     long int             reqbody_limit;
@@ -382,6 +410,7 @@ struct directory_config {
     apr_table_t         *of_mime_types;
     int                  of_mime_types_cleared;
     int                  of_limit_action;
+    int                  if_limit_action;
 
     const char          *debuglog_name;
     int                  debuglog_level;
@@ -454,17 +483,16 @@ struct directory_config {
     /* Content injection. */
     int                  content_injection_enabled;
 
-    /* PDF XSS Protection. */
-    int                  pdfp_enabled;
-    const char          *pdfp_secret;
-    int                  pdfp_timeout;
-    const char          *pdfp_token_name;
-    int                  pdfp_only_get;
-    int                  pdfp_method;
+    /* Stream Inspection */
+    int                 stream_inbody_inspection;
+    int                 stream_outbody_inspection;
 
     /* Geo Lookup */
     geo_db              *geo;
 
+    /* Gsb Lookup */
+    gsb_db              *gsb;
+
     /* Cache */
     int                  cache_trans;
     int                  cache_trans_incremental;
@@ -479,6 +507,8 @@ struct directory_config {
 
     /* Request character encoding. */
     const char          *request_encoding;
+    
+    int disable_backend_compression;
 };
 
 struct error_message {
@@ -522,6 +552,11 @@ struct msc_string {
     unsigned int             value_len;
 };
 
+struct msc_parm {
+    char                    *value;
+    unsigned int            pad_1;
+    unsigned int            pad_2;
+};
 
 /* Engine functions */
 
@@ -547,6 +582,8 @@ apr_status_t DSOLOCAL modsecurity_request_body_store(modsec_rec *msr,
 
 apr_status_t DSOLOCAL modsecurity_request_body_end(modsec_rec *msr, char **error_msg);
 
+apr_status_t DSOLOCAL modsecurity_request_body_to_stream(modsec_rec *msr, const char *buffer, int buflen, char **error_msg);
+
 apr_status_t DSOLOCAL modsecurity_request_body_retrieve_start(modsec_rec *msr, char **error_msg);
 
 apr_status_t DSOLOCAL modsecurity_request_body_retrieve_end(modsec_rec *msr);
