@@ -1,21 +1,17 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #ifndef _MSC_RELEASE_H_
 #define _MSC_RELEASE_H_
 
@@ -40,15 +36,9 @@
 #define PRINTF_ATTRIBUTE(a,b)
 #endif
 
-typedef struct modsec_build_type_rec {
-    const char * name;
-    int          val;
-} modsec_build_type_rec;
-extern DSOLOCAL modsec_build_type_rec modsec_build_type[];
-
 #define MODSEC_VERSION_MAJOR       "2"
-#define MODSEC_VERSION_MINOR       "5"
-#define MODSEC_VERSION_MAINT       "13"
+#define MODSEC_VERSION_MINOR       "6"
+#define MODSEC_VERSION_MAINT       "0"
 #define MODSEC_VERSION_TYPE        ""
 #define MODSEC_VERSION_RELEASE     ""
 
