@@ -1,26 +1,42 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include "modsecurity.h"
+#include "re.h"
 #include "msc_parsers.h"
 
 #define CHUNK_CAPACITY 8192
 
+
+/**
+ *
+ */
+void msre_engine_reqbody_processor_register(msre_engine *engine,
+    const char *name, void *fn_init, void *fn_process, void *fn_complete)
+{
+    msre_reqbody_processor_metadata *metadata = 
+        (msre_reqbody_processor_metadata *)apr_pcalloc(engine->mp,
+            sizeof(msre_reqbody_processor_metadata));
+    if (metadata == NULL) return;
+
+    metadata->name = name;
+    metadata->init = fn_init;
+    metadata->process = fn_process;
+    metadata->complete = fn_complete;
+    apr_table_setn(engine->reqbody_processors, name, (void *)metadata);
+}
+
 /**
  * Prepare to accept the request body (part 2).
  */
@@ -66,6 +82,7 @@ static apr_status_t modsecurity_request_body_start_init(modsec_rec *msr, char **
 apr_status_t modsecurity_request_body_start(modsec_rec *msr, char **error_msg) {
     *error_msg = NULL;
     msr->msc_reqbody_length = 0;
+    msr->stream_input_length = 0;
 
     /* Create a separate memory pool that will be used
      * to allocate structures from (not data, which is allocated
@@ -77,8 +94,24 @@ apr_status_t modsecurity_request_body_start(modsec_rec *msr, char **error_msg) {
 
     if (msr->msc_reqbody_processor != NULL) {
         char *my_error_msg = NULL;
+        msre_reqbody_processor_metadata *metadata =
+            (msre_reqbody_processor_metadata *)apr_table_get(msr->modsecurity->msre->reqbody_processors, msr->msc_reqbody_processor);
 
-        if (strcmp(msr->msc_reqbody_processor, "MULTIPART") == 0) {
+        if (metadata != NULL) {
+            if (   (metadata->init != NULL)
+                && (metadata->init(msr, &my_error_msg) < 0))
+            {
+                *error_msg = apr_psprintf(msr->mp,
+                                          "%s parsing error (init): %s",
+                                          msr->msc_reqbody_processor,
+                                          my_error_msg);
+                msr->msc_reqbody_error = 1;
+                msr->msc_reqbody_error_msg = my_error_msg;
+                msr_log(msr, 2, "%s", *error_msg);
+            }
+        }
+        // TODO: All these below need to be registered in the same way as above
+        else if (strcmp(msr->msc_reqbody_processor, "MULTIPART") == 0) {
             if (multipart_init(msr, &my_error_msg) < 0) {
                 *error_msg = apr_psprintf(msr->mp, "Multipart parsing error (init): %s", my_error_msg);
                 msr->msc_reqbody_error = 1;
@@ -86,8 +119,7 @@ apr_status_t modsecurity_request_body_start(modsec_rec *msr, char **error_msg) {
                 msr_log(msr, 2, "%s", *error_msg);
             }
         }
-        else
-        if (strcmp(msr->msc_reqbody_processor, "XML") == 0) {
+        else if (strcmp(msr->msc_reqbody_processor, "XML") == 0) {
             if (xml_init(msr, &my_error_msg) < 0) {
                 *error_msg = apr_psprintf(msr->mp, "XML parsing error (init): %s", my_error_msg);
                 msr->msc_reqbody_error = 1;
@@ -95,8 +127,7 @@ apr_status_t modsecurity_request_body_start(modsec_rec *msr, char **error_msg) {
                 msr_log(msr, 2, "%s", *error_msg);
             }
         }
-        else
-        if (strcmp(msr->msc_reqbody_processor, "URLENCODED") == 0) {
+        else if (strcmp(msr->msc_reqbody_processor, "URLENCODED") == 0) {
             /* Do nothing, URLENCODED processor does not support streaming yet. */
         }
         else {
@@ -268,10 +299,26 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
      * data to it first (but only if it did not report an
      * error on previous invocations).
      */
-    if ((msr->msc_reqbody_processor != NULL)&&(msr->msc_reqbody_error == 0)) {
+    if ((msr->msc_reqbody_processor != NULL) && (msr->msc_reqbody_error == 0)) {
         char *my_error_msg = NULL;
+        msre_reqbody_processor_metadata *metadata =
+            (msre_reqbody_processor_metadata *)apr_table_get(msr->modsecurity->msre->reqbody_processors, msr->msc_reqbody_processor);
 
-        if (strcmp(msr->msc_reqbody_processor, "MULTIPART") == 0) {
+        if (metadata != NULL) {
+            if (   (metadata->process != NULL)
+                && (metadata->process(msr, data, length, &my_error_msg) < 0))
+            {
+                *error_msg = apr_psprintf(msr->mp,
+                                          "%s parsing error: %s",
+                                          msr->msc_reqbody_processor,
+                                          my_error_msg);
+                msr->msc_reqbody_error = 1;
+                msr->msc_reqbody_error_msg = my_error_msg;
+                msr_log(msr, 2, "%s", *error_msg);
+            }
+        }
+        // TODO: All these below need to be registered in the same way as above
+        else if (strcmp(msr->msc_reqbody_processor, "MULTIPART") == 0) {
             /* The per-request data length counter will
              * be updated by the multipart parser.
              */
@@ -284,8 +331,7 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
                 msr_log(msr, 2, "%s", *error_msg);
             }
         }
-        else
-        if (strcmp(msr->msc_reqbody_processor, "XML") == 0) {
+        else if (strcmp(msr->msc_reqbody_processor, "XML") == 0) {
             /* Increase per-request data length counter. */
             msr->msc_reqbody_no_files_length += length;
 
@@ -297,8 +343,7 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
                 msr_log(msr, 2, "%s", *error_msg);
             }
         }
-        else
-        if (strcmp(msr->msc_reqbody_processor, "URLENCODED") == 0) {
+        else if (strcmp(msr->msc_reqbody_processor, "URLENCODED") == 0) {
             /* Increase per-request data length counter. */
             msr->msc_reqbody_no_files_length += length;
 
@@ -316,7 +361,7 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
 
     /* Check that we are not over the request body no files limit. */
     if (msr->msc_reqbody_no_files_length >= (unsigned long) msr->txcfg->reqbody_no_files_limit) {
-        return -5;
+            return -5;
     }
 
     /* Store data. */
@@ -324,7 +369,7 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
         return modsecurity_request_body_store_memory(msr, data, length, error_msg);
     }
     else
-    if (msr->msc_reqbody_storage == MSC_REQBODY_DISK) {
+        if (msr->msc_reqbody_storage == MSC_REQBODY_DISK) {
         return modsecurity_request_body_store_disk(msr, data, length, error_msg);
     }
 
@@ -334,6 +379,61 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
     return -1;
 }
 
+apr_status_t modsecurity_request_body_to_stream(modsec_rec *msr, const char *buffer, int buflen, char **error_msg) {
+    char *stream_input_body = NULL;
+    char *data = NULL;
+    int first_pkt = 0;
+
+    if(msr->stream_input_data == NULL)  {
+        msr->stream_input_data = (char *)calloc(sizeof(char), msr->stream_input_length + 1);
+        first_pkt = 1;
+    }
+    else    {
+
+        data = (char *)malloc(msr->stream_input_length + 1 - buflen);
+
+        if(data == NULL)
+            return -1;
+
+        memset(data, 0, msr->stream_input_length + 1 - buflen);
+        memcpy(data, msr->stream_input_data, msr->stream_input_length - buflen);
+
+        stream_input_body = (char *)realloc(msr->stream_input_data, msr->stream_input_length + 1);
+
+        if(stream_input_body == NULL)   {
+            free(msr->stream_input_data);
+            msr->stream_input_data = NULL;
+        }
+
+        msr->stream_input_data = (char *)stream_input_body;
+    }
+
+    if (msr->stream_input_data== NULL) {
+        if(data)    {
+            free(data);
+            data = NULL;
+        }
+        *error_msg = apr_psprintf(msr->mp, "Unable to allocate memory to hold request body on stream. Asked for %" APR_SIZE_T_FMT " bytes.",
+                msr->stream_input_length + 1);
+        return -1;
+    }
+
+    memset(msr->stream_input_data, 0, msr->stream_input_length+1);
+
+    if(first_pkt)   {
+        memcpy(msr->stream_input_data, buffer, msr->stream_input_length);
+    } else {
+        memcpy(msr->stream_input_data, data, msr->stream_input_length - buflen);
+        memcpy(msr->stream_input_data+(msr->stream_input_length - buflen), buffer, buflen);
+    }
+
+    if(data)    {
+        free(data);
+        data = NULL;
+    }
+
+    return 1;
+}
 /**
  * Replace a bunch of chunks holding a request body with a single large chunk.
  */
@@ -377,6 +477,7 @@ static apr_status_t modsecurity_request_body_end_raw(modsec_rec *msr, char **err
         }
     }
 
+
     /* Now free the memory used by the chunks. */
 
     chunks = (msc_data_chunk **)msr->msc_reqbody_chunks->elts;
@@ -424,6 +525,10 @@ static apr_status_t modsecurity_request_body_end_urlencoded(modsec_rec *msr, cha
         return -1;
     }
 
+    if (invalid_count) {
+        msr->urlencoded_error = 1;
+    }
+
     return 1;
 }
 
@@ -445,15 +550,33 @@ apr_status_t modsecurity_request_body_end(modsec_rec *msr, char **error_msg) {
     msr->msc_reqbody_read = 1;
 
     /* Finalise body processing. */
-    if ((msr->msc_reqbody_processor != NULL)&&(msr->msc_reqbody_error == 0)) {
+    if ((msr->msc_reqbody_processor != NULL) && (msr->msc_reqbody_error == 0)) {
         char *my_error_msg = NULL;
+        msre_reqbody_processor_metadata *metadata =
+            (msre_reqbody_processor_metadata *)apr_table_get(msr->modsecurity->msre->reqbody_processors, msr->msc_reqbody_processor);
 
-        if (strcmp(msr->msc_reqbody_processor, "MULTIPART") == 0) {
+        if (metadata != NULL) {
+            if (   (metadata->complete != NULL)
+                && (metadata->complete(msr, &my_error_msg) < 0))
+            {
+                *error_msg = apr_psprintf(msr->mp,
+                                          "%s parsing error (complete): %s",
+                                          msr->msc_reqbody_processor,
+                                          my_error_msg);
+                msr->msc_reqbody_error = 1;
+                msr->msc_reqbody_error_msg = my_error_msg;
+                msr_log(msr, 2, "%s", *error_msg);
+            }
+        }
+        // TODO: All these below need to be registered in the same way as above
+        else if (strcmp(msr->msc_reqbody_processor, "MULTIPART") == 0) {
             if (multipart_complete(msr, &my_error_msg) < 0) {
                 *error_msg = apr_psprintf(msr->mp, "Multipart parsing error: %s", my_error_msg);
                 msr->msc_reqbody_error = 1;
                 msr->msc_reqbody_error_msg = *error_msg;
-                msr_log(msr, 2, "%s", *error_msg);
+                if (msr->txcfg->debuglog_level >= 4) {
+                    msr_log(msr, 4, "%s", *error_msg);
+                }
                 return -1;
             }
 
@@ -465,12 +588,10 @@ apr_status_t modsecurity_request_body_end(modsec_rec *msr, char **error_msg) {
                 return -1;
             }
         }
-        else
-        if (strcmp(msr->msc_reqbody_processor, "URLENCODED") == 0) {
+        else if (strcmp(msr->msc_reqbody_processor, "URLENCODED") == 0) {
             return modsecurity_request_body_end_urlencoded(msr, error_msg);
         }
-        else
-        if (strcmp(msr->msc_reqbody_processor, "XML") == 0) {
+        else if (strcmp(msr->msc_reqbody_processor, "XML") == 0) {
             if (xml_complete(msr, &my_error_msg) < 0) {
                 *error_msg = apr_psprintf(msr->mp, "XML parser error: %s", my_error_msg);
                 msr->msc_reqbody_error = 1;
@@ -485,7 +606,7 @@ apr_status_t modsecurity_request_body_end(modsec_rec *msr, char **error_msg) {
     }
 
     /* Note the request body no files length. */
-    msr_log(msr, 4, "Reqest body no files length: %" APR_SIZE_T_FMT, msr->msc_reqbody_no_files_length);
+    msr_log(msr, 4, "Request body no files length: %" APR_SIZE_T_FMT, msr->msc_reqbody_no_files_length);
 
     return 1;
 }
