@@ -1,28 +1,24 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
+#include "modsecurity.h"
 #include <ctype.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "mod_security2_config.h"
 #include "msc_release.h"
 #include "msc_util.h"
 
@@ -73,6 +69,32 @@ static const short b64_reverse_t[256] = {
   -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2
 };
 
+static unsigned char *c2x(unsigned what, unsigned char *where);
+static unsigned char x2c(unsigned char *what);
+static unsigned char xsingle2c(unsigned char *what);
+
+/* \brief Remove escape char
+*
+* \param mptmp Pointer to the pool
+* \param input Pointer to input string
+* \param input_len Input data length
+*
+* \retval string On Success
+*/
+char *remove_escape(apr_pool_t *mptmp, const char *input, int input_len)  {
+    char *parm = apr_palloc(mptmp, input_len);
+    char *ret = parm;
+    int len = input_len;
+
+    for(; *input !='\0' && len >=0; input++, len--)    {
+        if(*input != '\\')    {
+            *parm++ = *input;
+        }
+    }
+
+    *parm = '\0';
+    return ret;
+}
 
 /**
  *
@@ -154,6 +176,66 @@ int decode_base64_ext(char *plain_text, const char *input, int input_len)
     return j;
 }
 
+/* \brief Convert const char to int
+*
+* \param c number string
+*
+* \retval n The converted number
+*/
+int convert_to_int(const char c)
+{
+   int n;
+   if ((c>='0') && (c<='9'))
+      n = c - '0';
+   else if ((c>='A') && (c<='F'))
+      n = c - 'A' + 10;
+   else if ((c>='a') && (c<='f'))
+      n = c - 'a' + 10;
+   else
+      n = 0;
+   return n;
+}
+
+/* \brief Set a match to tx.N
+*
+* \param msr Pointer to modsec resource
+* \param capture If ON match will be saved
+* \param match Pointer to captured string
+ *\parm tx_n The tx number to save the data
+*
+* \retval 0 On Sucess|Fail
+*/
+int set_match_to_tx(modsec_rec *msr, int capture, const char *match, int tx_n)  {
+
+    if (capture) {
+        msc_string *s = (msc_string *)apr_pcalloc(msr->mp, sizeof(msc_string));
+
+        if (s == NULL) return -1;
+
+        s->name = apr_psprintf(msr->mp,"%d", tx_n);
+        s->name_len = strlen(s->name);
+        s->value = apr_pstrdup(msr->mp, match);
+        if (s->value == NULL) return -1;
+        s->value_len = strlen(s->value);
+        apr_table_setn(msr->tx_vars, s->name, (void *)s);
+
+        if (msr->txcfg->debuglog_level >= 9) {
+            msr_log(msr, 9, "Added phrase match to TX.%d: %s",
+                    tx_n, log_escape_nq_ex(msr->mp, s->value, s->value_len));
+        }
+
+        /*
+        for(i = 0; i <= 9; i++) {
+            char buf[2];
+            apr_snprintf(buf, sizeof(buf), "%d", i);
+            apr_table_unset(msr->tx_vars, buf);
+        }
+        */
+    }
+
+    return 0;
+}
+
 
 /**
  * Parses a string that contains a name-value pair in the form "name=value".
@@ -356,7 +438,7 @@ int hex2bytes_inplace(unsigned char *data, int len) {
  * representation.
  */
 char *bytes2hex(apr_pool_t *pool, unsigned char *data, int len) {
-    static unsigned char b2hex[] = "0123456789abcdef";
+    static const unsigned char b2hex[] = "0123456789abcdef";
     char *hex = NULL;
     int i, j;
 
@@ -437,7 +519,7 @@ int remove_lf_crlf_inplace(char *text) {
  * into a proper byte. Handles uppercase and lowercase letters
  * but does not check for overflows.
  */
-unsigned char x2c(unsigned char *what) {
+static unsigned char x2c(unsigned char *what) {
     register unsigned char digit;
 
     digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A') + 10 : (what[0] - '0'));
@@ -450,7 +532,7 @@ unsigned char x2c(unsigned char *what) {
 /**
  * Converts a single hexadecimal digit into a decimal value.
  */
-unsigned char xsingle2c(unsigned char *what) {
+static unsigned char xsingle2c(unsigned char *what) {
     register unsigned char digit;
 
     digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A') + 10 : (what[0] - '0'));
@@ -573,7 +655,7 @@ char *strtolower_inplace(unsigned char *str) {
  * Converts a single byte into its hexadecimal representation.
  * Will overwrite two bytes at the destination.
  */
-unsigned char *c2x(unsigned what, unsigned char *where) {
+static unsigned char *c2x(unsigned what, unsigned char *where) {
     static const char c2x_table[] = "0123456789abcdef";
 
     what = what & 0xff;
@@ -583,24 +665,27 @@ unsigned char *c2x(unsigned what, unsigned char *where) {
     return where;
 }
 
+static char *_log_escape(apr_pool_t *p, const unsigned char *input,
+    unsigned long int input_length, int escape_quotes, int escape_colon, int escape_re);
+
+char *log_escape_re(apr_pool_t *mp, const char *text) {
+    return _log_escape(mp, (const unsigned char *)text, text ? strlen(text) : 0, 1, 1, 1);
+}
+
 char *log_escape(apr_pool_t *mp, const char *text) {
-    return _log_escape(mp, (const unsigned char *)text, text ? strlen(text) : 0, 1, 0);
+    return _log_escape(mp, (const unsigned char *)text, text ? strlen(text) : 0, 1, 0, 0);
 }
 
 char *log_escape_nq(apr_pool_t *mp, const char *text) {
-    return _log_escape(mp, (const unsigned char *)text, text ? strlen(text) : 0, 0, 0);
+    return _log_escape(mp, (const unsigned char *)text, text ? strlen(text) : 0, 0, 0, 0);
 }
 
 char *log_escape_ex(apr_pool_t *mp, const char *text, unsigned long int text_length) {
-    return _log_escape(mp, (const unsigned char *)text, text_length, 1, 0);
+    return _log_escape(mp, (const unsigned char *)text, text_length, 1, 0, 0);
 }
 
 char *log_escape_nq_ex(apr_pool_t *mp, const char *text, unsigned long int text_length) {
-    return _log_escape(mp, (const unsigned char *)text, text_length, 0, 0);
-}
-
-char *log_escape_header_name(apr_pool_t *mp, const char *text) {
-    return _log_escape(mp, (const unsigned char *)text, text ? strlen(text) : 0, 0, 1);
+    return _log_escape(mp, (const unsigned char *)text, text_length, 0, 0, 0);
 }
 
 char *log_escape_raw(apr_pool_t *mp, const unsigned char *text, unsigned long int text_length) {
@@ -669,8 +754,8 @@ char *log_escape_hex(apr_pool_t *mp, const unsigned char *text, unsigned long in
 /**
  * Transform input into a form safe for logging.
  */
-char *_log_escape(apr_pool_t *mp, const unsigned char *input, unsigned long int input_len,
-    int escape_quotes, int escape_colon)
+static char *_log_escape(apr_pool_t *mp, const unsigned char *input, unsigned long int input_len,
+    int escape_quotes, int escape_colon, int escape_re)
 {
     unsigned char *d = NULL;
     char *ret = NULL;
@@ -701,6 +786,70 @@ char *_log_escape(apr_pool_t *mp, const unsigned char *input, unsigned long int
                     *d++ = input[i];
                 }
                 break;
+            case '+' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = '+';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
+            case '.' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = '.';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
+            case ']' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = ']';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
+            case '[' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = '[';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
+            case '(' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = '(';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
+            case ')' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = ')';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
+            case '?' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = '?';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
+            case '/' :
+                if (escape_re) {
+                    *d++ = '\\';
+                    *d++ = '/';
+                } else {
+                    *d++ = input[i];
+                }
+                break;
             case '\b' :
                 *d++ = '\\';
                 *d++ = 'b';
@@ -1249,7 +1398,7 @@ int ansi_c_sequences_decode_inplace(unsigned char *input, int input_len) {
  *
  * IMP1 Assumes NUL-terminated
  */
-int normalise_path_inplace(unsigned char *input, int input_len, int win, int *changed) {
+int normalize_path_inplace(unsigned char *input, int input_len, int win, int *changed) {
     unsigned char *src;
     unsigned char *dst;
     unsigned char *end;
@@ -1626,3 +1775,32 @@ apr_fileperms_t mode2fileperms(int mode) {
     return perms;
 }
 
+/**
+ * Generate a single variable.
+ */
+char *construct_single_var(modsec_rec *msr, char *name) {
+    char *varname = NULL;
+    char *param = NULL;
+    msre_var *var = NULL;
+    msre_var *vx = NULL;
+    char *my_error_msg = NULL;
+
+    /* Extract variable name and its parameter from the script. */
+    varname = apr_pstrdup(msr->mp, name);
+    param = strchr(varname, '.');
+    if (param != NULL) {
+        *param = '\0';
+        param++;
+    }
+
+    /* Resolve variable. */
+    var = msre_create_var_ex(msr->mp, msr->modsecurity->msre,
+        varname, param, msr, &my_error_msg);
+    if (var == NULL) return NULL;
+
+    /* Generate variable. */
+    vx = generate_single_var(msr, var, NULL, NULL, msr->msc_rule_mptmp);
+    if (vx == NULL) return NULL;
+
+    return (char *)vx->value;
+}
