@@ -1,21 +1,17 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #ifndef _UTIL_H_
 #define _UTIL_H_
 
@@ -31,10 +27,12 @@
 
 #include "modsecurity.h"
 
-int DSOLOCAL normalise_path_inplace(unsigned char *input, int len, int win, int *changed);
+int DSOLOCAL normalize_path_inplace(unsigned char *input, int len, int win, int *changed);
 
 int DSOLOCAL parse_boolean(const char *input);
 
+char DSOLOCAL *remove_escape(apr_pool_t *mptmp, const char *input, int input_len);
+
 int DSOLOCAL parse_name_eq_value(apr_pool_t *mp, const char *input, char **name, char **value);
 
 char DSOLOCAL *url_encode(apr_pool_t *mp, char *input, unsigned int input_len, int *changed);
@@ -53,10 +51,6 @@ int DSOLOCAL is_token_char(unsigned char c);
 
 int DSOLOCAL remove_lf_crlf_inplace(char *text);
 
-unsigned char DSOLOCAL x2c(unsigned char *what);
-
-unsigned char DSOLOCAL xsingle2c(unsigned char *what);
-
 char DSOLOCAL *guess_tmp_dir(apr_pool_t *p);
 
 char DSOLOCAL *current_logtime(apr_pool_t *mp);
@@ -69,7 +63,7 @@ int DSOLOCAL msc_mkstemp(char *template);
 
 char DSOLOCAL *strtolower_inplace(unsigned char *str);
 
-unsigned char DSOLOCAL *c2x(unsigned what, unsigned char *where);
+char DSOLOCAL *log_escape_re(apr_pool_t *p, const char *text);
 
 char DSOLOCAL *log_escape(apr_pool_t *p, const char *text);
 
@@ -79,19 +73,18 @@ char DSOLOCAL *log_escape_ex(apr_pool_t *p, const char *text, unsigned long int
 
 char DSOLOCAL *log_escape_nq_ex(apr_pool_t *p, const char *text, unsigned long int text_length);
 
-char DSOLOCAL *log_escape_header_name(apr_pool_t *p, const char *text);
-
 char DSOLOCAL *log_escape_hex(apr_pool_t *mp, const unsigned char *text, unsigned long int text_length);
 
 char DSOLOCAL *log_escape_raw(apr_pool_t *mp, const unsigned char *text, unsigned long int text_length);
 
 char DSOLOCAL *log_escape_nul(apr_pool_t *mp, const unsigned char *text, unsigned long int text_length);
 
-char DSOLOCAL *_log_escape(apr_pool_t *p, const unsigned char *input,
-    unsigned long int input_length, int escape_quotes, int escape_colon);
-
 int DSOLOCAL decode_base64_ext(char *plain_text, const char *input, int input_len);
 
+int DSOLOCAL convert_to_int(const char c);
+
+int DSOLOCAL set_match_to_tx(modsec_rec *msr, int capture, const char *match, int tx_n);
+
 int DSOLOCAL js_decode_nonstrict_inplace(unsigned char *input, long int input_len);
 
 int DSOLOCAL urldecode_uni_nonstrict_inplace_ex(unsigned char *input, long int input_length, int * changed);
@@ -112,4 +105,8 @@ int DSOLOCAL css_decode_inplace(unsigned char *input, long int input_len);
 
 apr_fileperms_t DSOLOCAL mode2fileperms(int mode);
 
+char DSOLOCAL *construct_single_var(modsec_rec *msr, char *name);
+
+char DSOLOCAL *format_all_performance_variables(modsec_rec *msr, apr_pool_t *mp);
+
 #endif
