@@ -1,21 +1,17 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include "persist_dbm.h"
 #include "apr_sdbm.h"
 
@@ -174,15 +170,19 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
                     if (strcmp(key_to_expire, "__expire_KEY") == 0) {
                         expired = 1;
                     }
+                    
                     if (msr->txcfg->debuglog_level >= 9) {
                         msr_log(msr, 9, "Removing key \"%s\" from collection.", key_to_expire + 9);
                         msr_log(msr, 9, "Removing key \"%s\" from collection.", key_to_expire);
                     }
+                    
                     apr_table_unset(col, key_to_expire + 9);
                     apr_table_unset(col, key_to_expire);
+                    
                     if (msr->txcfg->debuglog_level >= 4) {
                         msr_log(msr, 4, "Removed expired variable \"%s\".", key_to_expire + 9);
                     }
+                    
                     break;
                 }
             }
@@ -225,7 +225,8 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
         }
 
         if (expired && (msr->txcfg->debuglog_level >= 9)) {
-            msr_log(msr, 9, "Collection expired (name \"%s\", key \"%s\").", col_name, log_escape_ex(msr->mp, col_key, col_key_len));
+            msr_log(msr, 9, "Collection expired (name \"%s\", key \"%s\").", col_name,
+                log_escape_ex(msr->mp, col_key, col_key_len));
         }
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Deleted collection (name \"%s\", key \"%s\").",
@@ -273,13 +274,13 @@ static apr_table_t *collection_retrieve_ex(apr_sdbm_t *existing_dbm, modsec_rec
 
     if (msr->txcfg->debuglog_level >= 4) {
         msr_log(msr, 4, "Retrieved collection (name \"%s\", key \"%s\").",
-                log_escape(msr->mp, col_name), log_escape_ex(msr->mp, col_key, col_key_len));
+            log_escape(msr->mp, col_name), log_escape_ex(msr->mp, col_key, col_key_len));
     }
 
     if ((existing_dbm == NULL) && dbm) {
         /* Should not ever get here */
         msr_log(msr, 1, "Internal Error: Collection remained open (name \"%s\", key \"%s\").",
-                log_escape(msr->mp, col_name), log_escape_ex(msr->mp, col_key, col_key_len));
+            log_escape(msr->mp, col_name), log_escape_ex(msr->mp, col_key, col_key_len));
 
         apr_sdbm_close(dbm);
     }
@@ -299,11 +300,18 @@ cleanup:
  *
  */
 apr_table_t *collection_retrieve(modsec_rec *msr, const char *col_name,
-    const char *col_key, int col_key_len) {
-    return collection_retrieve_ex(NULL, msr, col_name, col_key, col_key_len);
+    const char *col_key, int col_key_len)
+{
+    apr_time_t time_before = apr_time_now();
+    apr_table_t *rtable = NULL;
+    
+    rtable = collection_retrieve_ex(NULL, msr, col_name, col_key, col_key_len);
+    
+    msr->time_storage_read += apr_time_now() - time_before;
+    
+    return rtable;
 }
 
-
 /**
  *
  */
@@ -334,8 +342,8 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
 
     if (msr->txcfg->data_dir == NULL) {
         msr_log(msr, 1, "Unable to store collection (name \"%s\", key \"%s\"). Use "
-            "SecDataDir to define data directory first.",
-            log_escape_ex(msr->mp, var_name->value, var_name->value_len), log_escape_ex(msr->mp, var_key->value, var_key->value_len));
+            "SecDataDir to define data directory first.", log_escape_ex(msr->mp, var_name->value, var_name->value_len),
+            log_escape_ex(msr->mp, var_key->value, var_key->value_len));
         goto error;
     }
 
@@ -417,7 +425,8 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
     orig_col = (const apr_table_t *)apr_table_get(msr->collections_original, var_name->value);
     if (orig_col != NULL) {
         if (msr->txcfg->debuglog_level >= 9) {
-            msr_log(msr, 9, "Re-retrieving collection prior to store: %s", apr_psprintf(msr->mp, "%.*s", var_name->value_len, var_name->value));
+            msr_log(msr, 9, "Re-retrieving collection prior to store: %s",
+                apr_psprintf(msr->mp, "%.*s", var_name->value_len, var_name->value));
         }
 
         stored_col = (const apr_table_t *)collection_retrieve_ex(dbm, msr, var_name->value, var_key->value, var_key->value_len);
@@ -448,6 +457,7 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
 
                     var->value = apr_psprintf(msr->mp, "%d", newval);
                     var->value_len = strlen(var->value);
+                    
                     if (msr->txcfg->debuglog_level >= 9) {
                         msr_log(msr, 9, "Delta applied for %s.%s %d->%d (%d): %d + (%d) = %d [%s,%d]",
                         log_escape_ex(msr->mp, var_name->value, var_name->value_len),
@@ -530,7 +540,8 @@ int collection_store(modsec_rec *msr, apr_table_t *col) {
 
     if (msr->txcfg->debuglog_level >= 4) {
         msr_log(msr, 4, "Persisted collection (name \"%s\", key \"%s\").",
-            log_escape_ex(msr->mp, var_name->value, var_name->value_len), log_escape_ex(msr->mp, var_key->value, var_key->value_len));
+            log_escape_ex(msr->mp, var_name->value, var_name->value_len),
+            log_escape_ex(msr->mp, var_key->value, var_key->value_len));
     }
 
     return 0;
@@ -645,10 +656,11 @@ int collections_remove_stale(modsec_rec *msr, const char *col_name) {
                             log_escape_ex(msr->mp, key.dptr, key.dsize - 1), get_apr_error(msr->mp, rc));
                         goto error;
                     }
+                    
                     if (msr->txcfg->debuglog_level >= 4) {
                         msr_log(msr, 4, "Removed stale collection (name \"%s\", "
-                                "key \"%s\").", log_escape(msr->mp, col_name),
-                                log_escape_ex(msr->mp, key.dptr, key.dsize - 1));
+                            "key \"%s\").", log_escape(msr->mp, col_name),
+                            log_escape_ex(msr->mp, key.dptr, key.dsize - 1));
                     }
                 }
             }
