@@ -1,21 +1,17 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #ifndef _MSC_RE_H_
 #define _MSC_RE_H_
 
@@ -23,6 +19,7 @@
 #define POSITIVE_VALUE 1
 #define NEGATIVE_VALUE 2
 
+typedef struct msre_ipmatch msre_ipmatch;
 typedef struct msre_engine msre_engine;
 typedef struct msre_ruleset msre_ruleset;
 typedef struct msre_ruleset_internal msre_ruleset_internal;
@@ -34,6 +31,7 @@ typedef struct msre_tfn_metadata msre_tfn_metadata;
 typedef struct msre_actionset msre_actionset;
 typedef struct msre_action_metadata msre_action_metadata;
 typedef struct msre_action msre_action;
+typedef struct msre_reqbody_processor_metadata msre_reqbody_processor_metadata;
 typedef struct msre_cache_rec msre_cache_rec;
 
 #include "apr_general.h"
@@ -42,38 +40,24 @@ typedef struct msre_cache_rec msre_cache_rec;
 #include "msc_pcre.h"
 #include "persist_dbm.h"
 #include "apache2.h"
+#include "http_config.h"
 
 #if defined(WITH_LUA)
 #include "msc_lua.h"
 #endif
 
 /* Actions, variables, functions and operator functions */
-
-apr_status_t DSOLOCAL msre_action_setvar_execute(modsec_rec *msr, apr_pool_t *mptmp, msre_rule *rule, char *var_name, char *var_value);
+char DSOLOCAL *update_rule_target(cmd_parms *cmd, directory_config *dcfg, msre_ruleset *rset, const char *p1, const char *p2, const char *p3);
 
 apr_status_t DSOLOCAL collection_original_setvar(modsec_rec *msr, const char *col_name, const msc_string *orig_var);
 
 int DSOLOCAL expand_macros(modsec_rec *msr, msc_string *var, msre_rule *rule, apr_pool_t *mptmp);
 
-apr_status_t DSOLOCAL msre_parse_targets(msre_ruleset *ruleset, const char *text,
-    apr_array_header_t *arr, char **error_msg);
-
-apr_status_t DSOLOCAL msre_parse_actions(msre_engine *engine, msre_actionset *actionset,
-    const char *text, char **error_msg);
-
 msre_var_metadata DSOLOCAL *msre_resolve_var(msre_engine *engine, const char *name);
 
-msre_action_metadata DSOLOCAL *msre_resolve_action(msre_engine *engine, const char *name);
-
-msre_var DSOLOCAL *msre_create_var(msre_ruleset *ruleset, const char *name, const char *param,
-    modsec_rec *msr, char **error_msg);
-
 msre_var DSOLOCAL *msre_create_var_ex(apr_pool_t *pool, msre_engine *engine, const char *name, const char *param,
     modsec_rec *msr, char **error_msg);
 
-msre_action DSOLOCAL *msre_create_action(msre_engine *engine, const char *name,
-    const char *param, char **error_msg);
-
 int DSOLOCAL msre_parse_generic(apr_pool_t *pool, const char *text, apr_table_t *vartable,
     char **error_msg);
 
@@ -82,8 +66,10 @@ int DSOLOCAL rule_id_in_range(int ruleid, const char *range);
 msre_var DSOLOCAL *generate_single_var(modsec_rec *msr, msre_var *var, apr_array_header_t *tfn_arr,
     msre_rule *rule, apr_pool_t *mptmp);
 
+#if defined(WITH_LUA)
 apr_table_t DSOLOCAL *generate_multi_var(modsec_rec *msr, msre_var *var, apr_array_header_t *tfn_arr,
     msre_rule *rule, apr_pool_t *mptmp);
+#endif
 
 /* Structures with the corresponding functions */
 
@@ -93,6 +79,7 @@ struct msre_engine {
     apr_table_t             *operators;
     apr_table_t             *actions;
     apr_table_t             *tfns;
+    apr_table_t             *reqbody_processors;
 };
 
 msre_engine DSOLOCAL *msre_engine_create(apr_pool_t *parent_pool);
@@ -120,7 +107,7 @@ msre_ruleset DSOLOCAL *msre_ruleset_create(msre_engine *engine, apr_pool_t *mp);
 
 int DSOLOCAL msre_ruleset_rule_add(msre_ruleset *ruleset, msre_rule *rule, int phase);
 
-msre_rule DSOLOCAL *msre_ruleset_fetch_rule(msre_ruleset *ruleset, const char *id);
+msre_rule DSOLOCAL *msre_ruleset_fetch_rule(msre_ruleset *ruleset, const char *id, int offset);
 
 int DSOLOCAL msre_ruleset_rule_remove_with_exception(msre_ruleset *ruleset, rule_exception *re);
 
@@ -143,6 +130,12 @@ int DSOLOCAL msre_ruleset_phase_rule_remove_with_exception(msre_ruleset *ruleset
 #define RULE_TYPE_LUA           3  /* SecRuleScript */
 #endif
 
+struct msre_ipmatch  {
+    apr_ipsubnet_t *ipsubnet;
+    const char * address;
+    struct  msre_ipmatch *next;
+};
+
 struct msre_rule {
     apr_array_header_t      *targets;
     const char              *op_name;
@@ -170,6 +163,14 @@ struct msre_rule {
     /* Compiled Lua script. */
     msc_script              *script;
     #endif
+
+    ap_regex_t              *sub_regex;
+    char                    *sub_str;
+    char                    *re_str;
+    int                     re_precomp;
+    int                     escape_re;
+
+    msre_ipmatch            *ip_op;
 };
 
 char DSOLOCAL *msre_rule_generate_unparsed(apr_pool_t *pool, const msre_rule *rule, const char *targets, const char *args, const char *actions);
@@ -184,8 +185,6 @@ msre_rule DSOLOCAL *msre_rule_lua_create(msre_ruleset *ruleset,
     const char *actions, char **error_msg);
 #endif
 
-apr_status_t DSOLOCAL msre_rule_process(msre_rule *rule, modsec_rec *msr);
-
 #define VAR_SIMPLE              0    /* REQUEST_URI */
 #define VAR_LIST                1
 
@@ -257,10 +256,10 @@ struct msre_var_metadata {
 };
 
 struct msre_var {
-    const char              *name;
+    char                    *name;
     const char              *value;
     unsigned int             value_len;
-    const char              *param;
+    char                    *param;
     const void              *param_data;
     msre_var_metadata       *metadata;
     msc_regex_t             *param_regex;
@@ -280,6 +279,8 @@ struct msre_actionset {
     int                      severity;
     int                      phase;
     msre_rule               *rule;
+    int                     arg_min;
+    int                     arg_max;
 
     /* Flow */
     int                      is_chained;
@@ -303,8 +304,6 @@ struct msre_actionset {
     int                      block;
 };
 
-char DSOLOCAL *msre_actionset_generate_action_string(apr_pool_t *pool, const msre_actionset *actionset);
-
 void DSOLOCAL msre_engine_variable_register(msre_engine *engine, const char *name,
     unsigned int type, unsigned int argc_min, unsigned int argc_max,
     fn_var_validate_t validate, fn_var_generate_t generate,
@@ -362,6 +361,21 @@ struct msre_action {
     unsigned int             param_plusminus; /* ABSOLUTE_VALUE, POSITIVE_VALUE, NEGATIVE_VALUE */
 };
 
+void DSOLOCAL msre_engine_reqbody_processor_register(msre_engine *engine,
+    const char *name, void *fn_init, void *fn_process, void *fn_complete);
+
+typedef int (*fn_reqbody_processor_init_t)(modsec_rec *msr, char **error_msg);
+typedef int (*fn_reqbody_processor_process_t)(modsec_rec *msr, const char *buf,
+            unsigned int size, char **error_msg);
+typedef int (*fn_reqbody_processor_complete_t)(modsec_rec *msr, char **error_msg);
+
+struct msre_reqbody_processor_metadata {
+    const char                       *name;
+    fn_reqbody_processor_init_t       init;
+    fn_reqbody_processor_process_t    process;
+    fn_reqbody_processor_complete_t   complete;
+};
+
 /* -- MSRE Function Prototypes ---------------------------------------------- */
 
 msre_var_metadata DSOLOCAL *msre_resolve_var(msre_engine *engine, const char *name);
@@ -374,7 +388,6 @@ apr_status_t DSOLOCAL msre_parse_vars(msre_ruleset *ruleset, const char *text,
 
 char DSOLOCAL *msre_format_metadata(modsec_rec *msr, msre_actionset *actionset);
 
-
 /* -- Data Cache -- */
 
 struct msre_cache_rec {
