@@ -1,23 +1,22 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2008 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include "re.h"
 #include <ctype.h>
+#include "apr_lib.h"
+#include "apr_strmatch.h"
+
 
 /**
  * Register action with the engine.
@@ -102,6 +101,7 @@ msre_var *generate_single_var(modsec_rec *msr, msre_var *var, apr_array_header_t
     return rvar;
 }
 
+#if defined(WITH_LUA)
 /**
  *
  */
@@ -162,6 +162,7 @@ apr_table_t *generate_multi_var(modsec_rec *msr, msre_var *var, apr_array_header
 
     return tvartab;
 }
+#endif
 
 /**
  * Expands macros ("%{NAME}" entities) if present
@@ -253,11 +254,13 @@ int expand_macros(modsec_rec *msr, msc_string *var, msre_rule *rule, apr_pool_t
                         }
                     }
                 } else {
-                    msr_log(msr, 4, "Failed to resolve macro %%{%s%s%s}: %s",
-                        var_name,
-                        (var_value ? "." : ""),
-                        (var_value ? var_value : ""),
-                        my_error_msg);
+                    if (msr->txcfg->debuglog_level >= 4) {
+                        msr_log(msr, 4, "Failed to resolve macro %%{%s%s%s}: %s",
+                            var_name,
+                            (var_value ? "." : ""),
+                            (var_value ? var_value : ""),
+                            my_error_msg);
+                    }
                 }
             } else {
                 /* We could not identify a valid macro so add it as text. */
@@ -339,7 +342,8 @@ apr_status_t collection_original_setvar(modsec_rec *msr, const char *col_name, c
         var = (msc_string *)apr_table_get(table, var_name);
         if (var != NULL) {
             if (msr->txcfg->debuglog_level >= 9) {
-                msr_log(msr, 9, "Original collection variable: %s.%s = \"%s\"", col_name, var_name, log_escape_ex(msr->mp, orig_var->value, orig_var->value_len));
+                msr_log(msr, 9, "Original collection variable: %s.%s = \"%s\"", col_name, var_name,
+                    log_escape_ex(msr->mp, orig_var->value, orig_var->value_len));
             }
             return 1;
         }
@@ -359,7 +363,8 @@ apr_status_t collection_original_setvar(modsec_rec *msr, const char *col_name, c
     apr_table_setn(table, apr_pstrmemdup(msr->mp, var->name, var->name_len), (void *)var);
 
     if (msr->txcfg->debuglog_level >= 9) {
-        msr_log(msr, 9, "Recorded original collection variable: %s.%s = \"%s\"", col_name, var_name, log_escape_ex(msr->mp, var->value, var->value_len));
+        msr_log(msr, 9, "Recorded original collection variable: %s.%s = \"%s\"", col_name, var_name,
+            log_escape_ex(msr->mp, var->value, var->value_len));
     }
 
     return 0;
@@ -401,6 +406,34 @@ static apr_status_t msre_action_logdata_init(msre_engine *engine, msre_actionset
     return 1;
 }
 
+/* SanitizeMatchedBytes init */
+
+static apr_status_t msre_action_sanitizeMatchedBytes_init(msre_engine *engine,
+        msre_actionset *actionset, msre_action *action)
+{
+    char *parse_parm = NULL;
+    char *ac_param = NULL;
+    char *savedptr = NULL;
+    int arg_min = 0;
+    int arg_max = 0;
+
+    if (action->param != NULL && strlen(action->param) == 3)   {
+
+        ac_param = apr_pstrdup(engine->mp, action->param);
+        parse_parm = apr_strtok(ac_param,"/",&savedptr);
+
+        if(apr_isdigit(*parse_parm) && apr_isdigit(*savedptr))    {
+            arg_max = atoi(parse_parm);
+            arg_min = atoi(savedptr);
+        }
+    }
+
+    actionset->arg_min = arg_min;
+    actionset->arg_max = arg_max;
+
+    return 1;
+}
+
 /* severity */
 
 static apr_status_t msre_action_severity_init(msre_engine *engine,
@@ -545,7 +578,7 @@ static char *msre_action_proxy_validate(msre_engine *engine, msre_action *action
 }
 
 static apr_status_t msre_action_proxy_init(msre_engine *engine, msre_actionset *actionset,
-    msre_action *action)
+        msre_action *action)
 {
     actionset->intercept_action = ACTION_PROXY;
     actionset->intercept_uri = action->param;
@@ -554,7 +587,7 @@ static apr_status_t msre_action_proxy_init(msre_engine *engine, msre_actionset *
 }
 
 static apr_status_t msre_action_proxy_execute(modsec_rec *msr, apr_pool_t *mptmp,
-    msre_rule *rule, msre_action *action)
+        msre_rule *rule, msre_action *action)
 {
     msc_string *var = NULL;
 
@@ -572,7 +605,7 @@ static apr_status_t msre_action_proxy_execute(modsec_rec *msr, apr_pool_t *mptmp
 /* pass */
 
 static apr_status_t msre_action_pass_init(msre_engine *engine, msre_actionset *actionset,
-    msre_action *action)
+        msre_action *action)
 {
     actionset->intercept_action = ACTION_NONE;
     actionset->intercept_action_rec = action;
@@ -587,7 +620,7 @@ static char *msre_action_skip_validate(msre_engine *engine, msre_action *action)
 }
 
 static apr_status_t msre_action_skip_init(msre_engine *engine, msre_actionset *actionset,
-    msre_action *action)
+        msre_action *action)
 {
     actionset->skip_count = atoi(action->param);
     if (actionset->skip_count <= 0) actionset->skip_count = 1;
@@ -602,7 +635,7 @@ static char *msre_action_skipAfter_validate(msre_engine *engine, msre_action *ac
 }
 
 static apr_status_t msre_action_skipAfter_init(msre_engine *engine, msre_actionset *actionset,
-    msre_action *action)
+        msre_action *action)
 {
     actionset->skip_after = action->param;
     return 1;
@@ -700,6 +733,10 @@ static char *msre_action_ctl_validate(msre_engine *engine, msre_action *action)
         /* ENH nothing yet */
         return NULL;
     } else
+    if (strcasecmp(name, "ruleRemoveByTag") == 0) {
+        /* ENH nothing yet */
+        return NULL;
+    } else
     if (strcasecmp(name, "requestBodyAccess") == 0) {
         if (parse_boolean(value) == -1) {
             return apr_psprintf(engine->mp, "Invalid setting for ctl name "
@@ -781,14 +818,25 @@ static char *msre_action_ctl_validate(msre_engine *engine, msre_action *action)
         }
 
         return NULL;
-    }
-    else {
-        return apr_psprintf(engine->mp, "Invalid ctl name setting: %s", name);
-    }
+    } else
+        if  (strcasecmp(name, "ruleUpdateTargetById") == 0) {
+                char *parm = NULL;
+                char *savedptr = NULL;
+
+                parm = apr_strtok(value,";",&savedptr);
+
+                if(parm == NULL && savedptr == NULL)
+                    return apr_psprintf(engine->mp, "ruleUpdateTargetById must has at least id;append_value");
+
+                return NULL;
+        }
+        else {
+            return apr_psprintf(engine->mp, "Invalid ctl name setting: %s", name);
+        }
 }
 
 static apr_status_t msre_action_ctl_init(msre_engine *engine, msre_actionset *actionset,
-    msre_action *action)
+        msre_action *action)
 {
     /* Do nothing. */
     return 1;
@@ -819,12 +867,34 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
         if (strcasecmp(value, "detectiononly") == 0) {
             msr->txcfg->is_enabled = MODSEC_DETECTION_ONLY;
             msr->usercfg->is_enabled = MODSEC_DETECTION_ONLY;
+            msr->txcfg->if_limit_action = REQUEST_BODY_LIMIT_ACTION_PARTIAL;
+            msr->usercfg->if_limit_action = REQUEST_BODY_LIMIT_ACTION_PARTIAL;
+            msr->txcfg->of_limit_action = REQUEST_BODY_LIMIT_ACTION_PARTIAL;
+            msr->usercfg->of_limit_action = REQUEST_BODY_LIMIT_ACTION_PARTIAL;
+        }
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set ruleEngine to %s.", value);
         }
 
         return 1;
     } else
     if (strcasecmp(name, "ruleRemoveById") == 0) {
         *(const char **)apr_array_push(msr->removed_rules) = (const char *)apr_pstrdup(msr->mp, value);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Removed rule by id : %s.", value);
+        }
+
+        return 1;
+    } else
+    if (strcasecmp(name, "ruleRemoveByTag") == 0) {
+        *(const char **)apr_array_push(msr->removed_rules_tag) = (const char *)apr_pstrdup(msr->mp, value);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Removed rule by tag : %s.", value);
+        }
+
         return 1;
     } else
     if (strcasecmp(name, "requestBodyAccess") == 0) {
@@ -833,7 +903,10 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
         if (pv == -1) return -1;
         msr->txcfg->reqbody_access = pv;
         msr->usercfg->reqbody_access = pv;
-        msr_log(msr, 4, "Ctl: Set requestBodyAccess to %d.", pv);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set requestBodyAccess to %d.", pv);
+        }
 
         return 1;
     } else
@@ -848,13 +921,18 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
             msr->usercfg->reqbody_buffering = REQUEST_BODY_FORCEBUF_OFF;
         }
 
-        msr_log(msr, 4, "Ctl: Set requestBodyAccess to %d.", msr->txcfg->reqbody_buffering);
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set requestBodyAccess to %d.", msr->txcfg->reqbody_buffering);
+        }
 
         return 1;
     } else
     if (strcasecmp(name, "requestBodyProcessor") == 0) {
         msr->msc_reqbody_processor = value;
-        msr_log(msr, 4, "Ctl: Set requestBodyProcessor to %s.", value);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set requestBodyProcessor to %s.", value);
+        }
 
         return 1;
     } else
@@ -864,7 +942,10 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
         if (pv == -1) return -1;
         msr->txcfg->resbody_access = pv;
         msr->usercfg->resbody_access = pv;
-        msr_log(msr, 4, "Ctl: Set responseBodyAccess to %d.", pv);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set responseBodyAccess to %d.", pv);
+        }
 
         return 1;
     } else
@@ -884,7 +965,9 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
             msr->usercfg->auditlog_flag = AUDITLOG_RELEVANT;
         }
 
-        msr_log(msr, 4, "Ctl: Set auditEngine to %d.", msr->txcfg->auditlog_flag);
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set auditEngine to %d.", msr->txcfg->auditlog_flag);
+        }
 
         return 1;
     } else
@@ -920,14 +1003,20 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
         /* Set the new value. */
         msr->txcfg->auditlog_parts = new_value;
         msr->usercfg->auditlog_parts = new_value;
-        msr_log(msr, 4, "Ctl: Set auditLogParts to %s.", msr->txcfg->auditlog_parts);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set auditLogParts to %s.", msr->txcfg->auditlog_parts);
+        }
 
         return 1;
     } else
     if (strcasecmp(name, "debugLogLevel") == 0) {
         msr->txcfg->debuglog_level = atoi(value);
         msr->usercfg->debuglog_level = atoi(value);
-        msr_log(msr, 4, "Ctl: Set debugLogLevel to %d.", msr->txcfg->debuglog_level);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set debugLogLevel to %d.", msr->txcfg->debuglog_level);
+        }
 
         return 1;
     } else
@@ -938,6 +1027,10 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
         msr->txcfg->reqbody_limit = limit;
         msr->usercfg->reqbody_limit = limit;
 
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set requestBodyLimit to %ld.", limit);
+        }
+
         return 1;
     } else
     if (strcasecmp(name, "responseBodyLimit") == 0) {
@@ -947,6 +1040,28 @@ static apr_status_t msre_action_ctl_execute(modsec_rec *msr, apr_pool_t *mptmp,
         msr->txcfg->of_limit = limit;
         msr->usercfg->of_limit = limit;
 
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: Set responseBodyLimit to %ld.", limit);
+        }
+
+        return 1;
+    } else
+    if (strcasecmp(name, "ruleUpdateTargetById") == 0)  {
+        char *p1 = NULL, *p2 = NULL, *p3 = NULL;
+        char *savedptr = NULL;
+
+        p1 = apr_strtok(value,";",&savedptr);
+
+        p2 = apr_strtok(NULL,";",&savedptr);
+
+        p3 = apr_strtok(NULL,";",&savedptr);
+
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Ctl: ruleUpdateTargetById id=%s append=%s replace=%s", p1, p2, p3);
+        }
+
+        update_rule_target(NULL, NULL, rule->ruleset, p1, p2, p3);
+
         return 1;
     }
     else {
@@ -977,8 +1092,8 @@ static char *msre_action_xmlns_validate(msre_engine *engine, msre_action *action
     return NULL;
 }
 
-/* sanitiseArg */
-static apr_status_t msre_action_sanitiseArg_execute(modsec_rec *msr, apr_pool_t *mptmp,
+/* sanitizeArg */
+static apr_status_t msre_action_sanitizeArg_execute(modsec_rec *msr, apr_pool_t *mptmp,
     msre_rule *rule, msre_action *action)
 {
     const char *sargname = NULL;
@@ -994,7 +1109,7 @@ static apr_status_t msre_action_sanitiseArg_execute(modsec_rec *msr, apr_pool_t
         msc_arg *arg = (msc_arg *)telts[i].val;
 
         if (strcasecmp(sargname, arg->name) == 0) {
-            apr_table_addn(msr->arguments_to_sanitise, arg->name, (void *)arg);
+            apr_table_addn(msr->arguments_to_sanitize, arg->name, (void *)arg);
         }
     }
 
@@ -1005,8 +1120,8 @@ static apr_status_t msre_action_sanitiseArg_execute(modsec_rec *msr, apr_pool_t
 #define SANITISE_REQUEST_HEADER     2
 #define SANITISE_RESPONSE_HEADER    3
 
-/* sanitiseMatched */
-static apr_status_t msre_action_sanitiseMatched_execute(modsec_rec *msr, apr_pool_t *mptmp,
+/* sanitizeMatched */
+static apr_status_t msre_action_sanitizeMatched_execute(modsec_rec *msr, apr_pool_t *mptmp,
     msre_rule *rule, msre_action *action)
 {
     const char *sargname = NULL;
@@ -1045,8 +1160,10 @@ static apr_status_t msre_action_sanitiseMatched_execute(modsec_rec *msr, apr_poo
         type = SANITISE_RESPONSE_HEADER;
     }
     else {
-        msr_log(msr, 3, "sanitiseMatched: Don't know how to handle variable: %s",
-            mvar->name);
+        if (msr->txcfg->debuglog_level >= 3) {
+            msr_log(msr, 3, "sanitizeMatched: Don't know how to handle variable: %s",
+                mvar->name);
+        }
         return 0;
     }
 
@@ -1057,17 +1174,17 @@ static apr_status_t msre_action_sanitiseMatched_execute(modsec_rec *msr, apr_poo
             for (i = 0; i < tarr->nelts; i++) {
                 msc_arg *arg = (msc_arg *)telts[i].val;
                 if (strcasecmp(sargname, arg->name) == 0) {
-                    apr_table_addn(msr->arguments_to_sanitise, arg->name, (void *)arg);
+                    apr_table_addn(msr->arguments_to_sanitize, arg->name, (void *)arg);
                 }
             }
             break;
 
         case SANITISE_REQUEST_HEADER :
-            apr_table_set(msr->request_headers_to_sanitise, sargname, "1");
+            apr_table_set(msr->request_headers_to_sanitize, sargname, "1");
             break;
 
         case SANITISE_RESPONSE_HEADER :
-            apr_table_set(msr->response_headers_to_sanitise, sargname, "1");
+            apr_table_set(msr->response_headers_to_sanitize, sargname, "1");
             break;
 
         default :
@@ -1078,19 +1195,19 @@ static apr_status_t msre_action_sanitiseMatched_execute(modsec_rec *msr, apr_poo
     return 1;
 }
 
-/* sanitiseRequestHeader */
-static apr_status_t msre_action_sanitiseRequestHeader_execute(modsec_rec *msr, apr_pool_t *mptmp,
+/* sanitizeRequestHeader */
+static apr_status_t msre_action_sanitizeRequestHeader_execute(modsec_rec *msr, apr_pool_t *mptmp,
     msre_rule *rule, msre_action *action)
 {
-    apr_table_set(msr->request_headers_to_sanitise, action->param, "1");
+    apr_table_set(msr->request_headers_to_sanitize, action->param, "1");
     return 1;
 }
 
-/* sanitiseResponseHeader */
-static apr_status_t msre_action_sanitiseResponseHeader_execute(modsec_rec *msr, apr_pool_t *mptmp,
+/* sanitizeResponseHeader */
+static apr_status_t msre_action_sanitizeResponseHeader_execute(modsec_rec *msr, apr_pool_t *mptmp,
     msre_rule *rule, msre_action *action)
 {
-    apr_table_set(msr->response_headers_to_sanitise, action->param, "1");
+    apr_table_set(msr->response_headers_to_sanitize, action->param, "1");
     return 1;
 }
 
@@ -1205,10 +1322,14 @@ apr_status_t msre_action_setvar_execute(modsec_rec *msr, apr_pool_t *mptmp,
     target_col = msr->tx_vars;
     s = strstr(var_name, ".");
     if (s == NULL) {
-        msr_log(msr, 3, "Asked to set variable \"%s\", but no collection name specified. ",
-            log_escape(msr->mp, var_name));
-         return 0;
+        if (msr->txcfg->debuglog_level >= 3) {
+            msr_log(msr, 3, "Asked to set variable \"%s\", but no collection name specified. ",
+                log_escape(msr->mp, var_name));
+        }
+
+        return 0;
     }
+
     col_name = var_name;
     var_name = s + 1;
     *s = '\0';
@@ -1219,8 +1340,11 @@ apr_status_t msre_action_setvar_execute(modsec_rec *msr, apr_pool_t *mptmp,
     } else {
         target_col = (apr_table_t *)apr_table_get(msr->collections, col_name);
         if (target_col == NULL) {
-            msr_log(msr, 3, "Could not set variable \"%s.%s\" as the collection does not exist.",
-                log_escape(msr->mp, col_name), log_escape(msr->mp, var_name));
+            if (msr->txcfg->debuglog_level >= 3) {
+                msr_log(msr, 3, "Could not set variable \"%s.%s\" as the collection does not exist.",
+                    log_escape(msr->mp, col_name), log_escape(msr->mp, var_name));
+            }
+
             return 0;
         }
     }
@@ -1355,7 +1479,6 @@ static apr_status_t msre_action_setvar_parse(modsec_rec *msr, apr_pool_t *mptmp,
     return msre_action_setvar_execute(msr,mptmp,rule,var_name,var_value);
 }
 
-
 /* expirevar */
 static apr_status_t msre_action_expirevar_execute(modsec_rec *msr, apr_pool_t *mptmp,
     msre_rule *rule, msre_action *action)
@@ -1406,13 +1529,19 @@ static apr_status_t msre_action_expirevar_execute(modsec_rec *msr, apr_pool_t *m
 
         target_col = (apr_table_t *)apr_table_get(msr->collections, col_name);
         if (target_col == NULL) {
-            msr_log(msr, 3, "Could not expire variable \"%s.%s\" as the collection does not exist.",
-                log_escape(msr->mp, col_name), log_escape(msr->mp, var_name));
+            if (msr->txcfg->debuglog_level >= 3) {
+                msr_log(msr, 3, "Could not expire variable \"%s.%s\" as the collection does not exist.",
+                    log_escape(msr->mp, col_name), log_escape(msr->mp, var_name));
+            }
+            
             return 0;
         }
     } else {
-        msr_log(msr, 3, "Asked to expire variable \"%s\", but no collection name specified. ",
-            log_escape(msr->mp, var_name));
+        if (msr->txcfg->debuglog_level >= 3) {
+            msr_log(msr, 3, "Asked to expire variable \"%s\", but no collection name specified. ",
+                log_escape(msr->mp, var_name));
+        }
+        
         return 0;
     }
 
@@ -1437,8 +1566,10 @@ static apr_status_t msre_action_expirevar_execute(modsec_rec *msr, apr_pool_t *m
 
     apr_table_setn(target_col, var->name, (void *)var);
 
-    msr_log(msr, 4, "Variable \"%s.%s\" set to expire in %s seconds.", col_name,
-        var_name, var_value);
+    if (msr->txcfg->debuglog_level >= 4) {
+        msr_log(msr, 4, "Variable \"%s.%s\" set to expire in %s seconds.", col_name,
+           var_name, var_value);
+    }
 
     apr_table_set(msr->collections_dirty, col_name, "1");
 
@@ -1501,13 +1632,19 @@ static apr_status_t msre_action_deprecatevar_execute(modsec_rec *msr, apr_pool_t
 
         target_col = (apr_table_t *)apr_table_get(msr->collections, col_name);
         if (target_col == NULL) {
-            msr_log(msr, 3, "Could not deprecate variable \"%s.%s\" as the collection does "
-                "not exist.", log_escape(msr->mp, col_name), log_escape(msr->mp, var_name));
+            if (msr->txcfg->debuglog_level >= 3) {
+                msr_log(msr, 3, "Could not deprecate variable \"%s.%s\" as the collection does "
+                    "not exist.", log_escape(msr->mp, col_name), log_escape(msr->mp, var_name));
+            }
+            
             return 0;
         }
     } else {
-        msr_log(msr, 3, "Asked to deprecate variable \"%s\", but no collection name specified. ",
-            log_escape(msr->mp, var_name));
+        if (msr->txcfg->debuglog_level >= 3) {
+            msr_log(msr, 3, "Asked to deprecate variable \"%s\", but no collection name specified. ",
+                log_escape(msr->mp, var_name));
+        }
+        
         return 0;
     }
 
@@ -1555,9 +1692,11 @@ static apr_status_t msre_action_deprecatevar_execute(modsec_rec *msr, apr_pool_t
         var->value = apr_psprintf(msr->mp, "%ld", new_value);
         var->value_len = strlen(var->value);
 
-        msr_log(msr, 4, "Deprecated variable \"%s.%s\" from %ld to %ld (%" APR_TIME_T_FMT " seconds since "
-            "last update).", log_escape(msr->mp, col_name), log_escape(msr->mp, var_name),
-            current_value, new_value, (apr_time_t)(current_time - last_update_time));
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Deprecated variable \"%s.%s\" from %ld to %ld (%" APR_TIME_T_FMT " seconds since "
+                "last update).", log_escape(msr->mp, col_name), log_escape(msr->mp, var_name),
+                current_value, new_value, (apr_time_t)(current_time - last_update_time));
+        }
 
         apr_table_set(msr->collections_dirty, col_name, "1");
     } else {
@@ -1591,8 +1730,11 @@ static apr_status_t init_collection(modsec_rec *msr, const char *real_col_name,
 
     if (table == NULL) {
         /* Does not exist yet - create new. */
-        msr_log(msr, 4, "Creating collection (name \"%s\", key \"%s\").",
-            real_col_name, col_key);
+        
+        if (msr->txcfg->debuglog_level >= 4) {
+            msr_log(msr, 4, "Creating collection (name \"%s\", key \"%s\").",
+               real_col_name, col_key);
+        }
 
         table = apr_table_make(msr->mp, 24);
         if (table == NULL) return -1;
@@ -1681,12 +1823,14 @@ static apr_status_t init_collection(modsec_rec *msr, const char *real_col_name,
     /* Add the collection to the list. */
     apr_table_setn(msr->collections, apr_pstrdup(msr->mp, col_name), (void *)table);
 
-    if (strcmp(col_name, real_col_name) != 0) {
-        msr_log(msr, 4, "Added collection \"%s\" to the list as \"%s\".",
-            log_escape(msr->mp, real_col_name), log_escape(msr->mp, col_name));
-    } else {
-        msr_log(msr, 4, "Added collection \"%s\" to the list.",
-            log_escape(msr->mp, real_col_name));
+    if (msr->txcfg->debuglog_level >= 4) {
+        if (strcmp(col_name, real_col_name) != 0) {
+            msr_log(msr, 4, "Added collection \"%s\" to the list as \"%s\".",
+                log_escape(msr->mp, real_col_name), log_escape(msr->mp, col_name));
+        } else {
+            msr_log(msr, 4, "Added collection \"%s\" to the list.",
+                log_escape(msr->mp, real_col_name));
+        }
     }
 
     return 1;
@@ -2220,7 +2364,46 @@ void msre_engine_register_default_actions(msre_engine *engine) {
         ACTION_CGROUP_NONE,
         NULL,
         NULL,
-        msre_action_sanitiseArg_execute
+        msre_action_sanitizeArg_execute
+    );
+
+    /* sanitiseMatchedBytes */
+    msre_engine_action_register(engine,
+        "sanitiseMatchedBytes",
+        ACTION_NON_DISRUPTIVE,
+        0, 1,
+        NO_PLUS_MINUS,
+        ACTION_CARDINALITY_MANY,
+        ACTION_CGROUP_NONE,
+        NULL,
+        msre_action_sanitizeMatchedBytes_init,
+        msre_action_sanitizeMatched_execute
+    );
+
+    /* sanitizeMatchedBytes */
+    msre_engine_action_register(engine,
+        "sanitizeMatchedBytes",
+        ACTION_NON_DISRUPTIVE,
+        0, 1,
+        NO_PLUS_MINUS,
+        ACTION_CARDINALITY_MANY,
+        ACTION_CGROUP_NONE,
+        NULL,
+        msre_action_sanitizeMatchedBytes_init,
+        msre_action_sanitizeMatched_execute
+    );
+
+    /* sanitizeArg */
+    msre_engine_action_register(engine,
+        "sanitizeArg",
+        ACTION_NON_DISRUPTIVE,
+        1, 1,
+        NO_PLUS_MINUS,
+        ACTION_CARDINALITY_MANY,
+        ACTION_CGROUP_NONE,
+        NULL,
+        NULL,
+        msre_action_sanitizeArg_execute
     );
 
     /* sanitiseMatched */
@@ -2233,7 +2416,20 @@ void msre_engine_register_default_actions(msre_engine *engine) {
         ACTION_CGROUP_NONE,
         NULL,
         NULL,
-        msre_action_sanitiseMatched_execute
+        msre_action_sanitizeMatched_execute
+    );
+
+    /* sanitizeMatched */
+    msre_engine_action_register(engine,
+        "sanitizeMatched",
+        ACTION_NON_DISRUPTIVE,
+        0, 0,
+        NO_PLUS_MINUS,
+        ACTION_CARDINALITY_MANY,
+        ACTION_CGROUP_NONE,
+        NULL,
+        NULL,
+        msre_action_sanitizeMatched_execute
     );
 
     /* sanitiseRequestHeader */
@@ -2246,7 +2442,20 @@ void msre_engine_register_default_actions(msre_engine *engine) {
         ACTION_CGROUP_NONE,
         NULL,
         NULL,
-        msre_action_sanitiseRequestHeader_execute
+        msre_action_sanitizeRequestHeader_execute
+    );
+    
+    /* sanitizeRequestHeader */
+    msre_engine_action_register(engine,
+        "sanitizeRequestHeader",
+        ACTION_NON_DISRUPTIVE,
+        1, 1,
+        NO_PLUS_MINUS,
+        ACTION_CARDINALITY_MANY,
+        ACTION_CGROUP_NONE,
+        NULL,
+        NULL,
+        msre_action_sanitizeRequestHeader_execute
     );
 
     /* sanitiseResponseHeader */
@@ -2259,7 +2468,20 @@ void msre_engine_register_default_actions(msre_engine *engine) {
         ACTION_CGROUP_NONE,
         NULL,
         NULL,
-        msre_action_sanitiseResponseHeader_execute
+        msre_action_sanitizeResponseHeader_execute
+    );
+    
+    /* sanitizeResponseHeader */
+    msre_engine_action_register(engine,
+        "sanitizeResponseHeader",
+        ACTION_NON_DISRUPTIVE,
+        1, 1,
+        NO_PLUS_MINUS,
+        ACTION_CARDINALITY_MANY,
+        ACTION_CGROUP_NONE,
+        NULL,
+        NULL,
+        msre_action_sanitizeResponseHeader_execute
     );
 
     /* setenv */
@@ -2380,10 +2602,9 @@ void msre_engine_register_default_actions(msre_engine *engine) {
     );
 
     /* tag */
-    /* ENH: This should be ACTION_METADATA??? */
     msre_engine_action_register(engine,
         "tag",
-        ACTION_NON_DISRUPTIVE,
+        ACTION_METADATA,
         1, 1,
         NO_PLUS_MINUS,
         ACTION_CARDINALITY_MANY,
