@@ -1,21 +1,17 @@
 /*
- * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2010 Trustwave Holdings, Inc. (http://www.trustwave.com/)
- *
- * This product is released under the terms of the General Public Licence,
- * version 2 (GPLv2). Please refer to the file LICENSE (included with this
- * distribution) which contains the complete text of the licence.
- *
- * There are special exceptions to the terms and conditions of the GPL
- * as it is applied to this software. View the full text of the exception in
- * file MODSECURITY_LICENSING_EXCEPTION in the directory of this software
- * distribution.
- *
- * If any of the files related to licensing are missing or if you have any
- * other questions related to licensing please contact Trustwave Holdings, Inc.
- * directly using the email address support@trustwave.com.
- *
- */
+* ModSecurity for Apache 2.x, http://www.modsecurity.org/
+* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+*
+* You may not use this file except in compliance with
+* the License.  You may obtain a copy of the License at
+*
+*     http://www.apache.org/licenses/LICENSE-2.0
+*
+* If any of the files related to licensing are missing or if you have any
+* other questions related to licensing please contact Trustwave Holdings, Inc.
+* directly using the email address security@modsecurity.org.
+*/
+
 #include <ctype.h>
 
 #include "apr_md5.h"
@@ -25,28 +21,56 @@
 #include "re.h"
 #include "msc_util.h"
 
-/* Base64 tables used in decodeBase64Ext */
+/* cmdline */
 
-static const char b64_pad = '=';
+static int msre_fn_cmdline_execute(apr_pool_t *mptmp, unsigned char *input,
+        long int input_len, char **rval, long int *rval_len)
+{
+    int space = 0;
+    unsigned char *s = input;
 
-static const short b64_reverse_t[256] = {
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -1, -1, -2, -2, -1, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -1, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, 62, -2, -2, -2, 63,
-  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -2, -2, -2, -2, -2, -2,
-  -2, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
-  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -2, -2, -2, -2, -2,
-  -2, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
-  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2,
-  -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2, -2
-};
+    if (rval == NULL) return -1;
+
+    *rval = (char *)input;
+    /* Check characters */
+    for ( ; *input; input++ ) {
+        switch(*input) {
+            /* remove some characters */
+            case '"':
+            case '\'':
+            case '\\':
+            case '^':
+                continue;
+                /* replace some characters to space (only one) */
+            case ' ':
+            case ',':
+            case ';':
+            case '\t':
+            case '\r':
+            case '\n':
+                if (!space) {
+                    *s++ = ' ';
+                    space++;
+                }
+                break;
+            case '/':
+            case '(':
+                /* remove space before / or ( */
+                if (space) s--;
+                space = 0;
+                *s++ = *input;
+                break;
+                /* copy normal characters */
+            default :
+                *s++ = tolower(*input);
+                space = 0;
+        }
+    }
+
+    *s = 0;
+    *rval_len = strlen(*rval);
+    return 1;
+}
 
 /* lowercase */
 
@@ -474,27 +498,27 @@ static int msre_fn_escapeSeqDecode_execute(apr_pool_t *mptmp, unsigned char *inp
     return (*rval_len == input_len ? 0 : 1);
 }
 
-/* normalisePath */
+/* normalizePath */
 
-static int msre_fn_normalisePath_execute(apr_pool_t *mptmp, unsigned char *input,
+static int msre_fn_normalizePath_execute(apr_pool_t *mptmp, unsigned char *input,
     long int input_len, char **rval, long int *rval_len)
 {
     int changed;
 
-    *rval_len = normalise_path_inplace(input, input_len, 0, &changed);
+    *rval_len = normalize_path_inplace(input, input_len, 0, &changed);
     *rval = (char *)input;
 
     return changed;
 }
 
-/* normalisePathWin */
+/* normalizePathWin */
 
-static int msre_fn_normalisePathWin_execute(apr_pool_t *mptmp, unsigned char *input,
+static int msre_fn_normalizePathWin_execute(apr_pool_t *mptmp, unsigned char *input,
     long int input_len, char **rval, long int *rval_len)
 {
     int changed;
 
-    *rval_len = normalise_path_inplace(input, input_len, 1, &changed);
+    *rval_len = normalize_path_inplace(input, input_len, 1, &changed);
     *rval = (char *)input;
 
     return changed;
@@ -607,7 +631,7 @@ static int msre_fn_parityOdd7bit_execute(apr_pool_t *mptmp, unsigned char *input
 * \retval 0 On failure
 * \retval 1 On Success
 */
-static int msre_fn_decodeBase64Ext_execute(apr_pool_t *mptmp, unsigned char *input, long int input_len, char **rval, long int *rval_len)
+static int msre_fn_base64DecodeExt_execute(apr_pool_t *mptmp, unsigned char *input, long int input_len, char **rval, long int *rval_len)
 {
     *rval_len = input_len;
     *rval = apr_palloc(mptmp, *rval_len);
@@ -616,6 +640,7 @@ static int msre_fn_decodeBase64Ext_execute(apr_pool_t *mptmp, unsigned char *inp
     return *rval_len ? 1 : 0;
 }
 
+
 /* ------------------------------------------------------------------------------ */
 
 /**
@@ -727,13 +752,25 @@ void msre_engine_register_default_tfns(msre_engine *engine) {
     /* normalisePath */
     msre_engine_tfn_register(engine,
         "normalisePath",
-        msre_fn_normalisePath_execute
+        msre_fn_normalizePath_execute
+    );
+    
+    /* normalizePath */
+    msre_engine_tfn_register(engine,
+        "normalizePath",
+        msre_fn_normalizePath_execute
     );
 
     /* normalisePathWin */
     msre_engine_tfn_register(engine,
         "normalisePathWin",
-        msre_fn_normalisePathWin_execute
+        msre_fn_normalizePathWin_execute
+    );
+    
+    /* normalizePathWin */
+    msre_engine_tfn_register(engine,
+        "normalizePathWin",
+        msre_fn_normalizePathWin_execute
     );
 
     /* parityEven7bit */
@@ -796,6 +833,11 @@ void msre_engine_register_default_tfns(msre_engine *engine) {
         msre_fn_trimLeft_execute
     );
 
+    msre_engine_tfn_register(engine,
+        "cmdline",
+        msre_fn_cmdline_execute
+    );
+
     /* trimRight */
     msre_engine_tfn_register(engine,
         "trimRight",
@@ -820,9 +862,10 @@ void msre_engine_register_default_tfns(msre_engine *engine) {
         msre_fn_urlEncode_execute
     );
 
-    /* decodeBase64Ext */
+    /* base64DecodeExt */
     msre_engine_tfn_register(engine,
-        "decodeBase64Ext",
-        msre_fn_decodeBase64Ext_execute
+        "base64DecodeExt",
+        msre_fn_base64DecodeExt_execute
     );
+
 }
